@@ -77,14 +77,9 @@ BtSeederStateChoke::PeerEntry::operator<(const PeerEntry& rhs) const
   }
 }
 
-SharedHandle<Peer> BtSeederStateChoke::PeerEntry::getPeer() const
+void BtSeederStateChoke::PeerEntry::disableOptUnchoking()
 {
-  return _peer;
-}
-
-unsigned int BtSeederStateChoke::PeerEntry::getUploadSpeed() const
-{
-  return _uploadSpeed;
+  _peer->optUnchoking(false);
 }
 
 void BtSeederStateChoke::unchoke
@@ -100,11 +95,16 @@ void BtSeederStateChoke::unchoke
     _logger->info("RU: %s, ulspd=%u", (*r).getPeer()->ipaddr.c_str(),
 		  (*r).getUploadSpeed());
   }
-  if(_round == 2 && r != peers.end()) {
-    std::random_shuffle(r, peers.end(),
-			*(SimpleRandomizer::getInstance().get()));
-    (*r).getPeer()->optUnchoking(true);
-    _logger->info("POU: %s", (*r).getPeer()->ipaddr.c_str());
+
+  if(_round < 2) {
+    std::for_each(peers.begin(), peers.end(),
+		  std::mem_fun_ref(&PeerEntry::disableOptUnchoking));
+    if(r != peers.end()) {
+      std::random_shuffle(r, peers.end(),
+			  *(SimpleRandomizer::getInstance().get()));
+      (*r).getPeer()->optUnchoking(true);
+      _logger->info("POU: %s", (*r).getPeer()->ipaddr.c_str());
+    }
   }
 }
 
@@ -163,9 +163,4 @@ BtSeederStateChoke::executeChoke(const std::deque<SharedHandle<Peer> >& peerSet)
   }
 }
 
-const Time& BtSeederStateChoke::getLastRound() const
-{
-  return _lastRound;
-}
-
 } // namespace aria2
