@@ -33,7 +33,7 @@
  */
 /* copyright --> */
 #include "ByteArrayDiskWriter.h"
-#include "Util.h"
+#include "util.h"
 #include "A2STR.h"
 
 namespace aria2 {
