@@ -42,8 +42,8 @@
 #include "DHTMessageFactory.h"
 #include "DHTMessageDispatcher.h"
 #include "DHTMessageCallback.h"
-#include "PeerMessageUtil.h"
-#include "Util.h"
+#include "bittorrent_helper.h"
+#include "util.h"
 #include "bencode.h"
 
 namespace aria2 {
@@ -80,8 +80,8 @@ BDE DHTFindNodeReplyMessage::getResponse()
       i != _closestKNodes.end() && offset < DHTBucket::K*26; ++i) {
     SharedHandle<DHTNode> node = *i;
     memcpy(buffer+offset, node->getID(), DHT_ID_LENGTH);
-    if(PeerMessageUtil::createcompact(buffer+20+offset, node->getIPAddress(),
-				      node->getPort())) {
+    if(bittorrent::createcompact(buffer+20+offset, node->getIPAddress(),
+				 node->getPort())) {
       offset += 26;
     }
   }
@@ -103,7 +103,7 @@ void DHTFindNodeReplyMessage::setClosestKNodes(const std::deque<SharedHandle<DHT
 
 std::string DHTFindNodeReplyMessage::toStringOptional() const
 {
-  return "nodes="+Util::uitos(_closestKNodes.size());
+  return "nodes="+util::uitos(_closestKNodes.size());
 }
 
 } // namespace aria2
