@@ -37,9 +37,12 @@
 
 #include "PeerStorage.h"
 
+#include <map>
+
+#include "TimeA2.h"
+
 namespace aria2 {
 
-class BtContext;
 class Option;
 class Logger;
 class BtRuntime;
@@ -49,23 +52,26 @@ class PieceStorage;
 
 class DefaultPeerStorage : public PeerStorage {
 private:
-  SharedHandle<BtContext> btContext;
   SharedHandle<BtRuntime> _btRuntime;
   SharedHandle<PieceStorage> _pieceStorage;
   const Option* option;
   std::deque<SharedHandle<Peer> > peers;
   Logger* logger;
-  size_t maxPeerListSize;
   uint64_t removedPeerSessionDownloadLength;
   uint64_t removedPeerSessionUploadLength;
 
   BtSeederStateChoke* _seederStateChoke;
   BtLeecherStateChoke* _leecherStateChoke;
 
+  std::map<std::string, TransferStat> _peerTransferStatMap;
+
+  Time _lastTransferStatMapUpdated;
+
+  TransferStat _cachedTransferStat;
+
   bool isPeerAlreadyAdded(const SharedHandle<Peer>& peer);
 public:
-  DefaultPeerStorage(const SharedHandle<BtContext>& btContext,
-		     const Option* option);
+  DefaultPeerStorage(const Option* option);
 
   virtual ~DefaultPeerStorage();
 
@@ -87,16 +93,16 @@ public:
 
   virtual TransferStat calculateStat();
 
+  virtual void updateTransferStatFor(const SharedHandle<Peer>& peer);
+
+  virtual TransferStat getTransferStatFor(const SharedHandle<Peer>& peer);
+
   virtual void returnPeer(const SharedHandle<Peer>& peer);
 
   virtual bool chokeRoundIntervalElapsed();
 
   virtual void executeChoke();
 
-  void setMaxPeerListSize(size_t size) { this->maxPeerListSize = size; }
- 
-  size_t getMaxPeerListSize() const { return maxPeerListSize; }
-
   void deleteUnusedPeer(size_t delSize);
   
   void onErasingPeer(const SharedHandle<Peer>& peer);
