@@ -36,23 +36,31 @@
 #define _D_DL_RETRY_EX_H_
 #include "RecoverableException.h"
 
-class DlRetryEx : public RecoverableException {
+namespace aria2 {
+
+class DlRetryEx:public RecoverableException {
+protected:
+  virtual SharedHandle<Exception> copy() const
+  {
+    SharedHandle<Exception> e(new DlRetryEx(*this));
+    return e;
+  }
 public:
-  DlRetryEx(Exception* cause = 0):RecoverableException(cause) {}
-
-  DlRetryEx(const char* msg, ...) {
-    va_list ap;
-    va_start(ap, msg);
-    setMsg(msg, ap);
-    va_end(ap);
-  }
-
-  DlRetryEx(Exception* cause, const char* msg, ...):RecoverableException(cause) {
-    va_list ap;
-    va_start(ap, msg);
-    setMsg(msg, ap);
-    va_end(ap);
-  }
+  DlRetryEx(const char* file, int line, const std::string& msg):
+    RecoverableException(file, line, msg) {}
+  DlRetryEx(const char* file, int line, const std::string& msg,
+	    const Exception& cause):
+    RecoverableException(file, line, msg, cause) {}
+  DlRetryEx(const char* file, int line, const DlRetryEx& e):
+    RecoverableException(file, line, e) {}
+  DlRetryEx(const char* file, int line, const std::string& msg,
+	    downloadresultcode::RESULT code):
+    RecoverableException(file, line, msg, code) {}
 };
 
+#define DL_RETRY_EX(arg) DlRetryEx(__FILE__, __LINE__, arg)
+#define DL_RETRY_EX2(arg1, arg2) DlRetryEx(__FILE__, __LINE__, arg1, arg2)
+
+} // namespace aria2
+
 #endif // _D_DL_RETRY_EX_H_
