@@ -51,31 +51,36 @@
 #include "DiskAdaptor.h"
 #include "DownloadContext.h"
 #include "Option.h"
-#include "Util.h"
+#include "util.h"
 #include "Socket.h"
 #include "message.h"
 #include "prefs.h"
 #include "StringFormat.h"
 #include "Decoder.h"
+#include "RequestGroupMan.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigestHelper.h"
 #endif // ENABLE_MESSAGE_DIGEST
 
 namespace aria2 {
 
+static const size_t BUFSIZE = 16*1024;
+
 DownloadCommand::DownloadCommand(int cuid,
 				 const RequestHandle& req,
+				 const SharedHandle<FileEntry>& fileEntry,
 				 RequestGroup* requestGroup,
 				 DownloadEngine* e,
 				 const SocketHandle& s):
-  AbstractCommand(cuid, req, requestGroup, e, s)
+  AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
+  _buf(new unsigned char[BUFSIZE])
 #ifdef ENABLE_MESSAGE_DIGEST
   , _pieceHashValidationEnabled(false)
 #endif // ENABLE_MESSAGE_DIGEST
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   {
-    if(e->option->getAsBool(PREF_REALTIME_CHUNK_CHECKSUM)) {
+    if(getOption()->getAsBool(PREF_REALTIME_CHUNK_CHECKSUM)) {
       std::string algo = _requestGroup->getDownloadContext()->getPieceHashAlgo();
       if(MessageDigestContext::supports(algo)) {
 	_messageDigestContext.reset(new MessageDigestContext());
@@ -87,23 +92,21 @@ DownloadCommand::DownloadCommand(int cuid,
     }
   }
 #endif // ENABLE_MESSAGE_DIGEST
-  peerStat = _requestGroup->getSegmentMan()->getPeerStat(cuid);
-  if(peerStat.isNull()) {
-    peerStat.reset(new PeerStat(cuid, req->getHost(), req->getProtocol()));
-    _requestGroup->getSegmentMan()->registerPeerStat(peerStat);
-  }
+
+  peerStat = req->initPeerStat();
   peerStat->downloadStart();
+  _requestGroup->getSegmentMan()->registerPeerStat(peerStat);
 }
 
 DownloadCommand::~DownloadCommand() {
-  assert(peerStat.get());
   peerStat->downloadStop();
+  _requestGroup->getSegmentMan()->updateFastestPeerStat(peerStat);
+  delete [] _buf;
 }
 
 bool DownloadCommand::executeInternal() {
-  if(maxDownloadSpeedLimit > 0 &&
-     maxDownloadSpeedLimit <
-     _requestGroup->calculateStat().getDownloadSpeed()) {
+  if(e->_requestGroupMan->doesOverallDownloadSpeedExceed() ||
+     _requestGroup->doesDownloadSpeedExceed()) {
     e->commands.push_back(this);
     disableReadCheckSocket();
     return false;
@@ -111,15 +114,19 @@ bool DownloadCommand::executeInternal() {
   setReadCheckSocket(socket);
   SegmentHandle segment = _segments.front();
 
-  size_t BUFSIZE = 16*1024;
-  unsigned char buf[BUFSIZE];
   size_t bufSize;
-  if(segment->getLength() > 0 && segment->getLength()-segment->getWrittenLength() < BUFSIZE) {
-    bufSize = segment->getLength()-segment->getWrittenLength();
+  if(segment->getLength() > 0) {
+    if(static_cast<uint64_t>(segment->getPosition()+segment->getLength()) <=
+       static_cast<uint64_t>(_fileEntry->getLastOffset())) {
+      bufSize = std::min(segment->getLength()-segment->getWrittenLength(),
+			 BUFSIZE);
+    } else {
+      bufSize = std::min(static_cast<size_t>(_fileEntry->getLastOffset()-segment->getPositionToWrite()), BUFSIZE);
+    }
   } else {
     bufSize = BUFSIZE;
   }
-  socket->readData(buf, bufSize);
+  socket->readData(_buf, bufSize);
 
   const SharedHandle<DiskAdaptor>& diskAdaptor =
     _requestGroup->getPieceStorage()->getDiskAdaptor();
@@ -129,10 +136,10 @@ bool DownloadCommand::executeInternal() {
 
   std::string decoded;
   if(_transferEncodingDecoder.isNull()) {
-    bufFinal = buf;
+    bufFinal = _buf;
     bufSizeFinal = bufSize;
   } else {
-    decoded = _transferEncodingDecoder->decode(buf, bufSize);
+    decoded = _transferEncodingDecoder->decode(_buf, bufSize);
 
     bufFinal = reinterpret_cast<const unsigned char*>(decoded.c_str());
     bufSizeFinal = decoded.size();
@@ -156,73 +163,78 @@ bool DownloadCommand::executeInternal() {
   }
 
 #endif // ENABLE_MESSAGE_DIGEST
+  if(bufSizeFinal > 0) {
+    segment->updateWrittenLength(bufSizeFinal);
+  }
 
-  segment->updateWrittenLength(bufSizeFinal);
-  
   peerStat->updateDownloadLength(bufSize);
 
-  if(_requestGroup->getTotalLength() != 0 && bufSize == 0 &&
-     !socket->wantRead() && !socket->wantWrite()) {
-    throw DlRetryEx(EX_GOT_EOF);
-  }
-  bool segmentComplete = false;
+  _requestGroup->getSegmentMan()->updateDownloadSpeedFor(peerStat);
+
+  bool segmentPartComplete = false;
+  // Note that GrowSegment::complete() always returns false.
   if(_transferEncodingDecoder.isNull() && _contentEncodingDecoder.isNull()) {
-    if(segment->complete()) {
-      segmentComplete = true;
+    if(segment->complete() ||
+       segment->getPositionToWrite() == _fileEntry->getLastOffset()) {
+      segmentPartComplete = true;
     } else if(segment->getLength() == 0 && bufSize == 0 &&
 	      !socket->wantRead() && !socket->wantWrite()) {
-      segmentComplete = true;
+      segmentPartComplete = true;
     }
   } else if(!_transferEncodingDecoder.isNull() &&
-	    !_contentEncodingDecoder.isNull()) {
-    if(_transferEncodingDecoder->finished() &&
-       _contentEncodingDecoder->finished()) {
-      segmentComplete = true;
-    }
-  } else if(!_transferEncodingDecoder.isNull() &&
-	    _contentEncodingDecoder.isNull()) {
-    if(_transferEncodingDecoder->finished()) {
-      segmentComplete = true;
-    }
-  } else if(_transferEncodingDecoder.isNull() &&
-	    !_contentEncodingDecoder.isNull()) {
-    if(_contentEncodingDecoder->finished()) {
-      segmentComplete = true;
-    }
+	    (segment->complete() || segment->getPositionToWrite() == _fileEntry->getLastOffset())) {
+    segmentPartComplete = true;
+  } else if((_transferEncodingDecoder.isNull() ||
+	     _transferEncodingDecoder->finished()) &&
+	    (_contentEncodingDecoder.isNull() ||
+	     _contentEncodingDecoder->finished())) {
+    segmentPartComplete = true;
   }
 
-  if(segmentComplete) {
-    logger->info(MSG_SEGMENT_DOWNLOAD_COMPLETED, cuid);
+  if(!segmentPartComplete && bufSize == 0 &&
+     !socket->wantRead() && !socket->wantWrite()) {
+    throw DL_RETRY_EX(EX_GOT_EOF);
+  }
+
+  if(segmentPartComplete) {
+    if(segment->complete() || segment->getLength() == 0) {
+      // If segment->getLength() == 0, the server doesn't provide
+      // content length, but the client detected that download
+      // completed.
+      logger->info(MSG_SEGMENT_DOWNLOAD_COMPLETED, cuid);
 #ifdef ENABLE_MESSAGE_DIGEST
 
-    {
-      std::string expectedPieceHash =
-	_requestGroup->getDownloadContext()->getPieceHash(segment->getIndex());
-      if(_pieceHashValidationEnabled && !expectedPieceHash.empty()) {
-	if(segment->isHashCalculated()) {
-	  logger->debug("Hash is available! index=%lu",
-			static_cast<unsigned long>(segment->getIndex()));
-	  validatePieceHash(segment, expectedPieceHash, segment->getHashString());
+      {
+	std::string expectedPieceHash =
+	  _requestGroup->getDownloadContext()->getPieceHash(segment->getIndex());
+	if(_pieceHashValidationEnabled && !expectedPieceHash.empty()) {
+	  if(segment->isHashCalculated()) {
+	    logger->debug("Hash is available! index=%lu",
+			  static_cast<unsigned long>(segment->getIndex()));
+	    validatePieceHash(segment, expectedPieceHash, segment->getHashString());
+	  } else {
+	    _messageDigestContext->digestReset();
+	    validatePieceHash(segment, expectedPieceHash,
+			      MessageDigestHelper::digest
+			      (_messageDigestContext.get(),
+			       _requestGroup->getPieceStorage()->getDiskAdaptor(),
+			       segment->getPosition(),
+			       segment->getLength()));
+	  }
 	} else {
-	  _messageDigestContext->digestReset();
-	  validatePieceHash(segment, expectedPieceHash,
-			    MessageDigestHelper::digest
-			    (_messageDigestContext.get(),
-			     _requestGroup->getPieceStorage()->getDiskAdaptor(),
-			     segment->getPosition(),
-			     segment->getLength()));
+	  _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
 	}
-      } else {
-	_requestGroup->getSegmentMan()->completeSegment(cuid, segment);
       }
-    }
 
 #else // !ENABLE_MESSAGE_DIGEST
-
-    _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
-
+      _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
 #endif // !ENABLE_MESSAGE_DIGEST
-
+    } else {
+      // If segment is not cacnel here, in the next pipelining
+      // request, aria2 requests bad range
+      // [FileEntry->getLastOffset(), FileEntry->getLastOffset())
+      _requestGroup->getSegmentMan()->cancelSegment(cuid, segment);
+    }
     checkLowestDownloadSpeed();
     // this unit is going to download another segment.
     return prepareForNextSegment();
@@ -240,23 +252,34 @@ void DownloadCommand::checkLowestDownloadSpeed() const
   if(peerStat->getDownloadStartTime().elapsed(startupIdleTime)) {
     unsigned int nowSpeed = peerStat->calculateDownloadSpeed();
     if(lowestDownloadSpeedLimit > 0 &&  nowSpeed <= lowestDownloadSpeedLimit) {
-      throw DlAbortEx(StringFormat(EX_TOO_SLOW_DOWNLOAD_SPEED,
-				   nowSpeed,
-				   lowestDownloadSpeedLimit,
-				   req->getHost().c_str()).str());
+      throw DL_ABORT_EX2(StringFormat(EX_TOO_SLOW_DOWNLOAD_SPEED,
+				      nowSpeed,
+				      lowestDownloadSpeedLimit,
+				      req->getHost().c_str()).str(),
+			 downloadresultcode::TOO_SLOW_DOWNLOAD_SPEED);
     }
   }
 }
 
 bool DownloadCommand::prepareForNextSegment() {
   if(_requestGroup->downloadFinished()) {
+    // If this is a single file download, and file size becomes known
+    // just after downloading, set total length to FileEntry object
+    // here.
+    if(_requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
+      const SharedHandle<FileEntry>& fileEntry =
+	_requestGroup->getDownloadContext()->getFirstFileEntry();
+      if(fileEntry->getLength() == 0) {
+	fileEntry->setLength
+	  (_requestGroup->getPieceStorage()->getCompletedLength());
+      }
+    }
 #ifdef ENABLE_MESSAGE_DIGEST
     CheckIntegrityEntryHandle entry(new ChecksumCheckIntegrityEntry(_requestGroup));
     if(entry->isValidationReady()) {
       entry->initValidator();
-      CheckIntegrityCommand* command =
-	new CheckIntegrityCommand(e->newCUID(), _requestGroup, e, entry);
-      e->commands.push_back(command);
+      // TODO do we need cuttrailinggarbage here?
+      e->_checkIntegrityMan->pushEntry(entry);
     }
     e->setNoWait(true);
     e->setRefreshInterval(0);
@@ -267,6 +290,9 @@ bool DownloadCommand::prepareForNextSegment() {
     // segment.
     if(_segments.size() == 1) {
       SegmentHandle tempSegment = _segments.front();
+      if(!tempSegment->complete()) {
+	return prepareForRetry(0);
+      }
       SegmentHandle nextSegment =
 	_requestGroup->getSegmentMan()->getSegment(cuid,
 						   tempSegment->getIndex()+1);
@@ -294,12 +320,12 @@ void DownloadCommand::validatePieceHash(const SharedHandle<Segment>& segment,
   } else {
     logger->info(EX_INVALID_CHUNK_CHECKSUM,
 		 segment->getIndex(),
-		 Util::itos(segment->getPosition(), true).c_str(),
+		 util::itos(segment->getPosition(), true).c_str(),
 		 expectedPieceHash.c_str(),
 		 actualPieceHash.c_str());
     segment->clear();
     _requestGroup->getSegmentMan()->cancelSegment(cuid);
-    throw DlRetryEx
+    throw DL_RETRY_EX
       (StringFormat("Invalid checksum index=%d", segment->getIndex()).str());
   }
 }
