@@ -50,20 +50,23 @@
 #include "prefs.h"
 #include "HttpConnection.h"
 #include "Socket.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
 FtpInitiateConnectionCommand::FtpInitiateConnectionCommand
 (int cuid,
  const RequestHandle& req,
+ const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  DownloadEngine* e)
-  :InitiateConnectionCommand(cuid, req, requestGroup, e) {}
+  :InitiateConnectionCommand(cuid, req, fileEntry, requestGroup, e) {}
 
 FtpInitiateConnectionCommand::~FtpInitiateConnectionCommand() {}
 
 Command* FtpInitiateConnectionCommand::createNextCommand
-(const std::deque<std::string>& resolvedAddresses,
+(const std::string& hostname, const std::string& addr, uint16_t port,
+ const std::deque<std::string>& resolvedAddresses,
  const SharedHandle<Request>& proxyRequest)
 {
   Command* command;
@@ -71,45 +74,56 @@ Command* FtpInitiateConnectionCommand::createNextCommand
     std::map<std::string, std::string> options;
     SharedHandle<SocketCore> pooledSocket =
       e->popPooledSocket(options, req->getHost(), req->getPort());
+    std::string proxyMethod = resolveProxyMethod(req->getProtocol());
     if(pooledSocket.isNull()) {
-      logger->info(MSG_CONNECTING_TO_SERVER, cuid,
-		   proxyRequest->getHost().c_str(), proxyRequest->getPort());
+      logger->info(MSG_CONNECTING_TO_SERVER, cuid, addr.c_str(), port);
       socket.reset(new SocketCore());
-      socket->establishConnection(resolvedAddresses.front(),
-				  proxyRequest->getPort());
+      socket->establishConnection(addr, port);
       
-      if(e->option->get(PREF_PROXY_METHOD) == V_GET) {
+      if(proxyMethod == V_GET) {
+	// Use GET for FTP via HTTP proxy.
+	req->setMethod(Request::METHOD_GET);
 	SharedHandle<HttpConnection> hc
-	  (new HttpConnection(cuid, socket, e->option));
+	  (new HttpConnection(cuid, socket, getOption().get()));
 	
 	HttpRequestCommand* c =
-	  new HttpRequestCommand(cuid, req, _requestGroup, hc, e, socket);
+	  new HttpRequestCommand(cuid, req, _fileEntry,
+				 _requestGroup, hc, e, socket);
+	c->setConnectedAddr(hostname, addr, port);
 	c->setProxyRequest(proxyRequest);
 	command = c;
-      } else if(e->option->get(PREF_PROXY_METHOD) == V_TUNNEL) {
-	command = new FtpTunnelRequestCommand(cuid, req, _requestGroup, e,
-					      proxyRequest, socket);
+      } else if(proxyMethod == V_TUNNEL) {
+	FtpTunnelRequestCommand* c =
+	  new FtpTunnelRequestCommand(cuid, req, _fileEntry,
+				      _requestGroup, e,
+				      proxyRequest, socket);
+	c->setConnectedAddr(hostname, addr, port);
+	command = c;
       } else {
 	// TODO
-	throw DlAbortEx("ERROR");
+	throw DL_ABORT_EX("ERROR");
       }
     } else {
-      if(e->option->get(PREF_PROXY_METHOD) == V_TUNNEL) {
+      if(proxyMethod == V_TUNNEL) {
 	command =
-	  new FtpNegotiationCommand(cuid, req, _requestGroup, e, pooledSocket,
+	  new FtpNegotiationCommand(cuid, req, _fileEntry,
+				    _requestGroup, e, pooledSocket,
 				    FtpNegotiationCommand::SEQ_SEND_CWD,
 				    options["baseWorkingDir"]);
-      } else if(e->option->get(PREF_PROXY_METHOD) == V_GET) {
+      } else if(proxyMethod == V_GET) {
+	// Use GET for FTP via HTTP proxy.
+	req->setMethod(Request::METHOD_GET);
 	SharedHandle<HttpConnection> hc
-	  (new HttpConnection(cuid, pooledSocket, e->option));
+	  (new HttpConnection(cuid, pooledSocket, getOption().get()));
 	
 	HttpRequestCommand* c =
-	  new HttpRequestCommand(cuid, req, _requestGroup, hc, e, pooledSocket);
+	  new HttpRequestCommand(cuid, req, _fileEntry,
+				 _requestGroup, hc, e, pooledSocket);
 	c->setProxyRequest(proxyRequest);
 	command = c;
       } else {
 	// TODO
-	throw DlAbortEx("ERROR");
+	throw DL_ABORT_EX("ERROR");
       }
     }
   } else {
@@ -117,14 +131,18 @@ Command* FtpInitiateConnectionCommand::createNextCommand
     SharedHandle<SocketCore> pooledSocket =
       e->popPooledSocket(options, resolvedAddresses, req->getPort());
     if(pooledSocket.isNull()) {
-      logger->info(MSG_CONNECTING_TO_SERVER, cuid, req->getHost().c_str(),
-		   req->getPort());
+      logger->info(MSG_CONNECTING_TO_SERVER, cuid, addr.c_str(), port);
       socket.reset(new SocketCore());
-      socket->establishConnection(resolvedAddresses.front(), req->getPort());
-      command = new FtpNegotiationCommand(cuid, req, _requestGroup, e, socket);
+      socket->establishConnection(addr, port);
+      FtpNegotiationCommand* c =
+	new FtpNegotiationCommand(cuid, req, _fileEntry,
+				  _requestGroup, e, socket);
+      c->setConnectedAddr(hostname, addr, port);
+      command = c;
     } else {
       command =
-	new FtpNegotiationCommand(cuid, req, _requestGroup, e, pooledSocket,
+	new FtpNegotiationCommand(cuid, req, _fileEntry,
+				  _requestGroup, e, pooledSocket,
 				  FtpNegotiationCommand::SEQ_SEND_CWD,
 				  options["baseWorkingDir"]);
     }
