@@ -49,39 +49,43 @@
 #include "DlAbortEx.h"
 #include "message.h"
 #include "prefs.h"
-#include "Util.h"
+#include "util.h"
 #include "Option.h"
 #include "Logger.h"
 #include "Segment.h"
-#include "SingleFileDownloadContext.h"
+#include "DownloadContext.h"
 #include "DefaultBtProgressInfoFile.h"
 #include "RequestGroupMan.h"
 #include "DownloadFailureException.h"
-#include "ServerHost.h"
 #include "Socket.h"
 #include "StringFormat.h"
 #include "DiskAdaptor.h"
 #include "SegmentMan.h"
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
+#include "a2functional.h"
+#include "URISelector.h"
 
 namespace aria2 {
 
-FtpNegotiationCommand::FtpNegotiationCommand(int32_t cuid,
-					     const RequestHandle& req,
-					     RequestGroup* requestGroup,
-					     DownloadEngine* e,
-					     const SocketHandle& s,
-					     Seq seq,
-					     const std::string& baseWorkingDir):
-  AbstractCommand(cuid, req, requestGroup, e, s), sequence(seq),
+FtpNegotiationCommand::FtpNegotiationCommand
+(int32_t cuid,
+ const RequestHandle& req,
+ const SharedHandle<FileEntry>& fileEntry,
+ RequestGroup* requestGroup,
+ DownloadEngine* e,
+ const SocketHandle& s,
+ Seq seq,
+ const std::string& baseWorkingDir):
+  AbstractCommand(cuid, req, fileEntry, requestGroup, e, s), sequence(seq),
   ftp(new FtpConnection(cuid, socket, req,
-			e->getAuthConfigFactory()->createAuthConfig(req),
-			e->option))
+			e->getAuthConfigFactory()->createAuthConfig
+			(req, requestGroup->getOption().get()),
+			getOption().get()))
 {
   ftp->setBaseWorkingDir(baseWorkingDir);
   if(seq == SEQ_RECV_GREETING) {
-    setTimeout(e->option->getAsInt(PREF_CONNECT_TIMEOUT));
+    setTimeout(getOption()->getAsInt(PREF_CONNECT_TIMEOUT));
   }
   disableReadCheckSocket();
   setWriteCheckSocket(socket);
@@ -95,28 +99,28 @@ bool FtpNegotiationCommand::executeInternal() {
     return prepareForRetry(0);
   } else if(sequence == SEQ_NEGOTIATION_COMPLETED) {
     FtpDownloadCommand* command =
-      new FtpDownloadCommand(cuid, req, _requestGroup, ftp, e, dataSocket, socket);
-    command->setMaxDownloadSpeedLimit(e->option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
-    command->setStartupIdleTime(e->option->getAsInt(PREF_STARTUP_IDLE_TIME));
-    command->setLowestDownloadSpeedLimit(e->option->getAsInt(PREF_LOWEST_SPEED_LIMIT));
-    if(!_requestGroup->isSingleHostMultiConnectionEnabled()) {
-      SharedHandle<ServerHost> sv =
-	_requestGroup->searchServerHost(req->getHost());
-      if(!sv.isNull()) {
-	_requestGroup->removeURIWhoseHostnameIs(sv->getHostname());
-      }
+      new FtpDownloadCommand
+      (cuid, req, _fileEntry, _requestGroup, ftp, e, dataSocket, socket);
+    command->setStartupIdleTime(getOption()->getAsInt(PREF_STARTUP_IDLE_TIME));
+    command->setLowestDownloadSpeedLimit(getOption()->getAsInt(PREF_LOWEST_SPEED_LIMIT));
+    if(!_fileEntry->isSingleHostMultiConnectionEnabled()) {
+      _fileEntry->removeURIWhoseHostnameIs(req->getHost());
     }
+    _requestGroup->getURISelector()->tuneDownloadCommand
+      (_fileEntry->getRemainingUris(), command);
     e->commands.push_back(command);
     return true;
   } else if(sequence == SEQ_HEAD_OK || sequence == SEQ_DOWNLOAD_ALREADY_COMPLETED) {
     return true;
   } else if(sequence == SEQ_FILE_PREPARATION) {
-    if(e->option->getAsBool(PREF_FTP_PASV)) {
+    if(getOption()->getAsBool(PREF_FTP_PASV)) {
       sequence = SEQ_SEND_PASV;
     } else {
       sequence = SEQ_PREPARE_SERVER_SOCKET;
     }
     return false;
+  } else if(sequence == SEQ_EXIT) {
+    return true;
   } else {
     e->commands.push_back(this);
     return false;
@@ -124,8 +128,12 @@ bool FtpNegotiationCommand::executeInternal() {
 }
 
 bool FtpNegotiationCommand::recvGreeting() {
-  checkIfConnectionEstablished(socket);
-  setTimeout(e->option->getAsInt(PREF_TIMEOUT));
+  if(!checkIfConnectionEstablished
+     (socket, _connectedHostname, _connectedAddr, _connectedPort)) {
+    sequence = SEQ_EXIT;
+    return false;
+  }
+  setTimeout(_requestGroup->getTimeout());
   //socket->setBlockingMode();
   disableWriteCheckSocket();
   setReadCheckSocket(socket);
@@ -135,7 +143,7 @@ bool FtpNegotiationCommand::recvGreeting() {
     return false;
   }
   if(status != 220) {
-    throw DlAbortEx(EX_CONNECTION_FAILED);
+    throw DL_ABORT_EX(EX_CONNECTION_FAILED);
   }
   sequence = SEQ_SEND_USER;
 
@@ -164,7 +172,7 @@ bool FtpNegotiationCommand::recvUser() {
     sequence = SEQ_SEND_PASS;
     break;
   default:
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
   return true;
 }
@@ -185,7 +193,7 @@ bool FtpNegotiationCommand::recvPass() {
     return false;
   }
   if(status != 230) {
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
   sequence = SEQ_SEND_TYPE;
   return true;
@@ -207,7 +215,7 @@ bool FtpNegotiationCommand::recvType() {
     return false;
   }
   if(status != 200) {
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
   sequence = SEQ_SEND_PWD;
   return true;
@@ -232,7 +240,7 @@ bool FtpNegotiationCommand::recvPwd()
     return false;
   }
   if(status != 257) {
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
   ftp->setBaseWorkingDir(pwd);
   logger->info("CUID#%d - base working directory is '%s'", cuid, pwd.c_str());
@@ -260,9 +268,13 @@ bool FtpNegotiationCommand::recvCwd() {
   if(status != 250) {
     poolConnection();
     _requestGroup->increaseAndValidateFileNotFoundCount();
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    if (status == 550)
+      throw DL_ABORT_EX2(MSG_RESOURCE_NOT_FOUND,
+			 downloadresultcode::RESOURCE_NOT_FOUND);
+    else
+      throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
-  if(e->option->getAsBool(PREF_REMOTE_TIME)) {
+  if(getOption()->getAsBool(PREF_REMOTE_TIME)) {
     sequence = SEQ_SEND_MDTM;
   } else {
     sequence = SEQ_SEND_SIZE;
@@ -321,36 +333,51 @@ bool FtpNegotiationCommand::sendSize() {
 
 bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
 {
-  SingleFileDownloadContextHandle dctx =
-    dynamic_pointer_cast<SingleFileDownloadContext>(_requestGroup->getDownloadContext());
-  dctx->setTotalLength(totalLength);
-  dctx->setFilename(Util::urldecode(req->getFile()));
+  _fileEntry->setLength(totalLength);
+  if(_fileEntry->getPath().empty()) {
+    _fileEntry->setPath
+      (strconcat(getDownloadContext()->getDir(),
+		 "/", util::urldecode(req->getFile())));
+  }
   _requestGroup->preDownloadProcessing();
   if(e->_requestGroupMan->isSameFileBeingDownloaded(_requestGroup)) {
-    throw DownloadFailureException
+    throw DOWNLOAD_FAILURE_EXCEPTION
       (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-		    _requestGroup->getFilePath().c_str()).str());
+		    _requestGroup->getFirstFilePath().c_str()).str());
   }
   if(totalLength == 0) {
 
-    if(e->option->getAsBool(PREF_FTP_PASV)) {
+    if(getOption()->getAsBool(PREF_FTP_PASV)) {
       sequence = SEQ_SEND_PASV;
     } else {
       sequence = SEQ_PREPARE_SERVER_SOCKET;
     }
-    _requestGroup->initPieceStorage();
-    if(dctx->knowsTotalLength() &&
-       _requestGroup->downloadFinishedByFileLength()) {
+
+    if(getOption()->getAsBool(PREF_DRY_RUN)) {
+      _requestGroup->initPieceStorage();
+      onDryRunFileFound();
+      return false;
+    }
+
+    if(_requestGroup->downloadFinishedByFileLength()) {
+      _requestGroup->initPieceStorage();
+      _requestGroup->getPieceStorage()->markAllPiecesDone();
       sequence = SEQ_DOWNLOAD_ALREADY_COMPLETED;
 
+      logger->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
+		     _requestGroup->getGID(),
+		     _requestGroup->getFirstFilePath().c_str());
+
       poolConnection();
 
       return false;
     }
+
     _requestGroup->shouldCancelDownloadForSafety();
+    _requestGroup->initPieceStorage();
     _requestGroup->getPieceStorage()->getDiskAdaptor()->initAndOpenFile();
 
-    if(dctx->knowsTotalLength()) {
+    if(getDownloadContext()->knowsTotalLength()) {
       sequence = SEQ_DOWNLOAD_ALREADY_COMPLETED;
       poolConnection();
       return false;
@@ -358,12 +385,28 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
 
     return true;
   } else {
+    _requestGroup->adjustFilename
+      (SharedHandle<BtProgressInfoFile>(new DefaultBtProgressInfoFile
+					(_requestGroup->getDownloadContext(),
+					 SharedHandle<PieceStorage>(),
+					 getOption().get())));
     _requestGroup->initPieceStorage();
 
-    BtProgressInfoFileHandle infoFile(new DefaultBtProgressInfoFile(_requestGroup->getDownloadContext(), _requestGroup->getPieceStorage(), e->option));
+    if(getOption()->getAsBool(PREF_DRY_RUN)) {
+      onDryRunFileFound();
+      return false;
+    }
+
+    BtProgressInfoFileHandle infoFile(new DefaultBtProgressInfoFile(_requestGroup->getDownloadContext(), _requestGroup->getPieceStorage(), getOption().get()));
     if(!infoFile->exists() && _requestGroup->downloadFinishedByFileLength()) {
+      _requestGroup->getPieceStorage()->markAllPiecesDone();
+
       sequence = SEQ_DOWNLOAD_ALREADY_COMPLETED;
       
+      logger->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
+		     _requestGroup->getGID(),
+		     _requestGroup->getFirstFilePath().c_str());
+
       poolConnection();
       
       return false;
@@ -386,8 +429,8 @@ bool FtpNegotiationCommand::recvSize() {
   if(status == 213) {
 
     if(size > INT64_MAX) {
-      throw DlAbortEx
-	(StringFormat(EX_TOO_LARGE_FILE, Util::uitos(size, true).c_str()).str());
+      throw DL_ABORT_EX
+	(StringFormat(EX_TOO_LARGE_FILE, util::uitos(size, true).c_str()).str());
     }
     if(_requestGroup->getPieceStorage().isNull()) {
 
@@ -395,7 +438,7 @@ bool FtpNegotiationCommand::recvSize() {
       return onFileSizeDetermined(size);
 
     } else {
-      _requestGroup->validateTotalLength(size);
+      _requestGroup->validateTotalLength(_fileEntry->getLength(), size);
     }
 
   } else {
@@ -406,17 +449,14 @@ bool FtpNegotiationCommand::recvSize() {
     // command, resuming and segmented downloading are disabled when the first
     // contacted FTP server doesn't support it.
     if(_requestGroup->getPieceStorage().isNull()) {
-      SingleFileDownloadContextHandle dctx =
-	dynamic_pointer_cast<SingleFileDownloadContext>
-	(_requestGroup->getDownloadContext());
-      dctx->markTotalLengthIsUnknown();
+      getDownloadContext()->markTotalLengthIsUnknown();
       return onFileSizeDetermined(0);
 
     }
     // TODO Skipping RequestGroup::validateTotalLength(0) here will allow
     // wrong file to be downloaded if user-specified URL is wrong.
   }
-  if(e->option->getAsBool(PREF_FTP_PASV)) {
+  if(getOption()->getAsBool(PREF_FTP_PASV)) {
     sequence = SEQ_SEND_PASV;
   } else {
     sequence = SEQ_PREPARE_SERVER_SOCKET;
@@ -453,7 +493,7 @@ bool FtpNegotiationCommand::recvPort() {
     return false;
   }
   if(status != 200) {
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
   sequence = SEQ_SEND_REST;
   return true;
@@ -477,7 +517,7 @@ bool FtpNegotiationCommand::recvPasv() {
     return false;
   }
   if(status != 227) {
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
   // make a data connection to the server.
   logger->info(MSG_CONNECTING_TO_SERVER, cuid,
@@ -519,7 +559,7 @@ bool FtpNegotiationCommand::recvRest(const SharedHandle<Segment>& segment) {
   // then throw exception here.
   if(status != 350) {
     if(!segment.isNull() && segment->getPositionToWrite() != 0) {
-      throw DlAbortEx("FTP server doesn't support resuming.");
+      throw DL_ABORT_EX("FTP server doesn't support resuming.");
     }
   }
   sequence = SEQ_SEND_RETR;
@@ -543,9 +583,13 @@ bool FtpNegotiationCommand::recvRetr() {
   }
   if(status != 150 && status != 125) {
     _requestGroup->increaseAndValidateFileNotFoundCount();
-    throw DlAbortEx(StringFormat(EX_BAD_STATUS, status).str());
+    if (status == 550)
+      throw DL_ABORT_EX2(MSG_RESOURCE_NOT_FOUND,
+			 downloadresultcode::RESOURCE_NOT_FOUND);
+    else
+      throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, status).str());
   }
-  if(e->option->getAsBool(PREF_FTP_PASV)) {
+  if(getOption()->getAsBool(PREF_FTP_PASV)) {
     sequence = SEQ_NEGOTIATION_COMPLETED;
     return false;
   } else {
@@ -561,7 +605,7 @@ bool FtpNegotiationCommand::waitConnection()
   disableReadCheckSocket();
   setReadCheckSocket(socket);
   dataSocket.reset(serverSocket->acceptConnection());
-  //dataSocket->setBlockingMode();
+  dataSocket->setNonBlockingMode();
   sequence = SEQ_NEGOTIATION_COMPLETED;
   return false;
 }
@@ -629,11 +673,18 @@ bool FtpNegotiationCommand::processSequence(const SegmentHandle& segment) {
 
 void FtpNegotiationCommand::poolConnection() const
 {
-  if(e->option->getAsBool(PREF_FTP_REUSE_CONNECTION)) {
+  if(getOption()->getAsBool(PREF_FTP_REUSE_CONNECTION)) {
     std::map<std::string, std::string> options;
     options["baseWorkingDir"] = ftp->getBaseWorkingDir();
     e->poolSocket(req, isProxyDefined(),  socket, options);
   }
 }
 
+void FtpNegotiationCommand::onDryRunFileFound()
+{
+  _requestGroup->getPieceStorage()->markAllPiecesDone();
+  poolConnection();
+  sequence = SEQ_HEAD_OK;
+}
+
 } // namespace aria2
