@@ -34,18 +34,33 @@
 /* copyright --> */
 #include "FtpTunnelResponseCommand.h"
 #include "FtpNegotiationCommand.h"
+#include "Request.h"
+#include "HttpConnection.h"
+#include "HttpRequest.h"
+#include "Segment.h"
+#include "Socket.h"
+#include "DownloadContext.h"
 
-FtpTunnelResponseCommand::FtpTunnelResponseCommand(int cuid,
-						   const RequestHandle& req,
-						   RequestGroup* requestGroup,
-						   const HttpConnectionHandle& httpConnection,
-						   DownloadEngine* e,
-						   const SocketHandle& s)
-  :AbstractProxyResponseCommand(cuid, req, requestGroup, httpConnection,e, s) {}
+namespace aria2 {
+
+FtpTunnelResponseCommand::FtpTunnelResponseCommand
+(int cuid,
+ const RequestHandle& req,
+ const SharedHandle<FileEntry>& fileEntry,
+ RequestGroup* requestGroup,
+ const HttpConnectionHandle& httpConnection,
+ DownloadEngine* e,
+ const SocketHandle& s)
+  :AbstractProxyResponseCommand(cuid, req, fileEntry, requestGroup,
+				httpConnection,e, s)
+{}
 
 FtpTunnelResponseCommand::~FtpTunnelResponseCommand() {}
 
 Command* FtpTunnelResponseCommand::getNextCommand()
 {
-  return new FtpNegotiationCommand(cuid, req, _requestGroup, e, socket);
+  return new FtpNegotiationCommand(cuid, req, _fileEntry,
+				   _requestGroup, e, socket);
 }
+
+} // namespace aria2
