@@ -46,18 +46,20 @@
 #include "HttpResponse.h"
 #include "HttpHeader.h"
 #include "Range.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
 HttpDownloadCommand::HttpDownloadCommand
 (int cuid,
  const RequestHandle& req,
+ const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  const SharedHandle<HttpResponse>& httpResponse,
  const HttpConnectionHandle& httpConnection,
  DownloadEngine* e,
  const SocketHandle& socket)
-  :DownloadCommand(cuid, req, requestGroup, e, socket),
+  :DownloadCommand(cuid, req, fileEntry, requestGroup, e, socket),
    _httpResponse(httpResponse),
    _httpConnection(httpConnection) {}
 
@@ -67,11 +69,12 @@ bool HttpDownloadCommand::prepareForNextSegment() {
   bool downloadFinished = _requestGroup->downloadFinished();
   if(req->isPipeliningEnabled() && !downloadFinished) {
     HttpRequestCommand* command =
-      new HttpRequestCommand(cuid, req, _requestGroup, _httpConnection, e,
+      new HttpRequestCommand(cuid, req, _fileEntry,
+			     _requestGroup, _httpConnection, e,
 			     socket);
     // Set proxy request here. aria2 sends the HTTP request specialized for
     // proxy.
-    if(e->option->get(PREF_PROXY_METHOD) == V_GET) {
+    if(resolveProxyMethod(req->getProtocol()) == V_GET) {
       command->setProxyRequest(createProxyRequest());
     }
     e->commands.push_back(command);
@@ -81,8 +84,7 @@ bool HttpDownloadCommand::prepareForNextSegment() {
        (req->isKeepAliveEnabled() &&
 	((!_transferEncodingDecoder.isNull() &&
 	  _requestGroup->downloadFinished()) ||
-	 (uint64_t)_segments.front()->getPositionToWrite() ==
-	 _requestGroup->getTotalLength()))) {
+	 _fileEntry->getLastOffset() == _segments.front()->getPositionToWrite()))) {
       e->poolSocket(req, isProxyDefined(), socket);
     }
     // The request was sent assuming that server supported pipelining, but
@@ -91,11 +93,15 @@ bool HttpDownloadCommand::prepareForNextSegment() {
     // of the response with the end byte of segment.
     // If it is the same, HTTP negotiation is necessary for the next request.
     if(!req->isPipeliningEnabled() && req->isPipeliningHint() &&
-       !_segments.empty() && !downloadFinished) {
+       !downloadFinished) {
       const SharedHandle<Segment>& segment = _segments.front();
-      if(segment->getPosition()+segment->getLength() ==
-	 static_cast<uint64_t>(_httpResponse->getHttpHeader()->
-			       getRange()->getEndByte()+1)) {
+
+      off_t lastOffset =_fileEntry->gtoloff
+	(std::min(static_cast<off_t>(segment->getPosition()+segment->getLength()),
+		  _fileEntry->getLastOffset()));
+      
+      if(lastOffset ==
+	 _httpResponse->getHttpHeader()->getRange()->getEndByte()+1) {
 	return prepareForRetry(0);
       }
     }
