@@ -48,21 +48,23 @@
 #include "Socket.h"
 #include "prefs.h"
 #include "a2functional.h"
-#include "Util.h"
+#include "util.h"
 #include "CookieStorage.h"
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
 HttpRequestCommand::HttpRequestCommand
 (int cuid,
  const RequestHandle& req,
+ const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  const HttpConnectionHandle& httpConnection,
  DownloadEngine* e,
  const SocketHandle& s)
-  :AbstractCommand(cuid, req, requestGroup, e, s),
+  :AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
    _httpConnection(httpConnection)
 {
   setTimeout(getOption()->getAsInt(PREF_CONNECT_TIMEOUT));
@@ -74,6 +76,7 @@ HttpRequestCommand::~HttpRequestCommand() {}
 
 static SharedHandle<HttpRequest>
 createHttpRequest(const SharedHandle<Request>& req,
+		  const SharedHandle<FileEntry>& fileEntry,
 		  const SharedHandle<Segment>& segment,
 		  uint64_t totalLength,
 		  const SharedHandle<Option>& option,
@@ -85,19 +88,12 @@ createHttpRequest(const SharedHandle<Request>& req,
   HttpRequestHandle httpRequest(new HttpRequest());
   httpRequest->setUserAgent(option->get(PREF_USER_AGENT));
   httpRequest->setRequest(req);
+  httpRequest->setFileEntry(fileEntry);
   httpRequest->setSegment(segment);
-  httpRequest->setEntityLength(totalLength);
   httpRequest->addHeader(option->get(PREF_HEADER));
   httpRequest->setCookieStorage(cookieStorage);
-  httpRequest->setAuthConfigFactory(authConfigFactory);
+  httpRequest->setAuthConfigFactory(authConfigFactory, option.get());
   httpRequest->setProxyRequest(proxyRequest);
-  if(!rg->getAcceptFeatures().empty()) {
-    const std::deque<std::string>& acceptFeatures = rg->getAcceptFeatures();
-    std::string acceptFeaturesHeader = "Accept-Features: ";
-    std::for_each(acceptFeatures.begin(), acceptFeatures.end(),
-		  StringAppend(acceptFeaturesHeader, ","));
-    httpRequest->addHeader(Util::trim(acceptFeaturesHeader, ","));
-  }
   httpRequest->addAcceptType(rg->getAcceptTypes().begin(),
 			     rg->getAcceptTypes().end());
 
@@ -116,11 +112,16 @@ bool HttpRequestCommand::executeInternal() {
     }
   }
   if(_httpConnection->sendBufferIsEmpty()) {
-    checkIfConnectionEstablished(socket);
+    if(!checkIfConnectionEstablished
+       (socket, _connectedHostname, _connectedAddr, _connectedPort)) {
+      return true;
+    }
 
     if(_segments.empty()) {
       HttpRequestHandle httpRequest
-	(createHttpRequest(req, SharedHandle<Segment>(),
+	(createHttpRequest(req,
+			   _fileEntry,
+			   SharedHandle<Segment>(),
 			   _requestGroup->getTotalLength(),
 			   getOption(),
 			   _requestGroup,
@@ -133,7 +134,9 @@ bool HttpRequestCommand::executeInternal() {
 	const SegmentHandle& segment = *itr;
 	if(!_httpConnection->isIssued(segment)) {
 	  HttpRequestHandle httpRequest
-	    (createHttpRequest(req, segment,
+	    (createHttpRequest(req,
+			       _fileEntry,
+			       segment,
 			       _requestGroup->getTotalLength(),
 			       getOption(),
 			       _requestGroup,
@@ -148,7 +151,8 @@ bool HttpRequestCommand::executeInternal() {
     _httpConnection->sendPendingData();
   }
   if(_httpConnection->sendBufferIsEmpty()) {
-    Command* command = new HttpResponseCommand(cuid, req, _requestGroup,
+    Command* command = new HttpResponseCommand(cuid, req, _fileEntry,
+					       _requestGroup,
 					       _httpConnection, e, socket);
     e->commands.push_back(command);
     return true;
