@@ -35,36 +35,55 @@
 #ifndef _D_PEER_CONNECTION_H_
 #define _D_PEER_CONNECTION_H_
 
-#include "Option.h"
-#include "Socket.h"
-#include "Logger.h"
 #include "common.h"
+#include "SharedHandle.h"
+#include "SocketBuffer.h"
+#include <unistd.h>
 
-// we assume maximum length of incoming message is "piece" message with 16KB
-// data. Messages beyond that size are dropped.
-#define MAX_PAYLOAD_LEN (9+16*1024)
+namespace aria2 {
+
+class Logger;
+class SocketCore;
+class ARC4Encryptor;
+class ARC4Decryptor;
+
+// The maximum length of payload. Messages beyond that length are
+// dropped.
+#define MAX_PAYLOAD_LEN (16*1024+128)
 
 class PeerConnection {
 private:
   int32_t cuid;
-  SocketHandle socket;
-  const Option* option;
-  const Logger* logger;
+  SharedHandle<SocketCore> socket;
+  Logger* logger;
 
-  char resbuf[MAX_PAYLOAD_LEN];
-  int32_t resbufLength;
-  int32_t currentPayloadLength;
-  char lenbuf[4];
-  int32_t lenbufLength;
+  unsigned char resbuf[MAX_PAYLOAD_LEN];
+  size_t resbufLength;
+  size_t currentPayloadLength;
+  unsigned char lenbuf[4];
+  size_t lenbufLength;
+
+  SocketBuffer _socketBuffer;
+
+  bool _encryptionEnabled;
+  SharedHandle<ARC4Encryptor> _encryptor;
+  SharedHandle<ARC4Decryptor> _decryptor;
+
+  bool _prevPeek;
+
+  void readData(unsigned char* data, size_t& length, bool encryption);
+
+  ssize_t sendData(const unsigned char* data, size_t length, bool encryption);
 
 public:
-  PeerConnection(int32_t cuid, const SocketHandle& socket, const Option* op);
+  PeerConnection(int32_t cuid, const SharedHandle<SocketCore>& socket);
+
   ~PeerConnection();
   
   // Returns the number of bytes written
-  int32_t sendMessage(const unsigned char* data, int32_t dataLength);
+  ssize_t sendMessage(const unsigned char* data, size_t dataLength);
 
-  bool receiveMessage(unsigned char* data, int32_t& dataLength);
+  bool receiveMessage(unsigned char* data, size_t& dataLength);
 
   /**
    * Returns true if a handshake message is fully received, otherwise returns
@@ -72,9 +91,21 @@ public:
    * In both cases, 'msg' is filled with received bytes and the filled length
    * is assigned to 'length'.
    */
-  bool receiveHandshake(unsigned char* data, int32_t& dataLength, bool peek = false);
+  bool receiveHandshake(unsigned char* data, size_t& dataLength, bool peek = false);
+
+  void enableEncryption(const SharedHandle<ARC4Encryptor>& encryptor,
+			const SharedHandle<ARC4Decryptor>& decryptor);
+
+  void presetBuffer(const unsigned char* data, size_t length);
+
+  bool sendBufferIsEmpty() const;
+  
+  ssize_t sendPendingData();
 };
 
 typedef SharedHandle<PeerConnection> PeerConnectionHandle;
 typedef WeakHandle<PeerConnection> PeerConnectionWeakHandle;
+
+} // namespace aria2
+
 #endif // _D_PEER_CONNECTION_H_
