@@ -42,11 +42,12 @@
 #include "Socket.h"
 #include "Logger.h"
 #include "Peer.h"
-#include "BtContext.h"
 #include "BtRuntime.h"
 #include "PeerStorage.h"
 #include "PieceStorage.h"
 #include "PeerConnection.h"
+#include "RequestGroup.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
@@ -55,21 +56,21 @@ PeerInitiateConnectionCommand::PeerInitiateConnectionCommand
  RequestGroup* requestGroup,
  const PeerHandle& peer,
  DownloadEngine* e,
- const SharedHandle<BtContext>& btContext,
  const SharedHandle<BtRuntime>& btRuntime,
  bool mseHandshakeEnabled)
   :
   PeerAbstractCommand(cuid, peer, e),
-  RequestGroupAware(requestGroup),
-  _btContext(btContext),
+  _requestGroup(requestGroup),
   _btRuntime(btRuntime),
   _mseHandshakeEnabled(mseHandshakeEnabled)
 {
   _btRuntime->increaseConnections();
+  _requestGroup->increaseNumCommand();
 }
 
 PeerInitiateConnectionCommand::~PeerInitiateConnectionCommand()
 {
+  _requestGroup->decreaseNumCommand();
   _btRuntime->decreaseConnections();
 }
 
@@ -80,7 +81,7 @@ bool PeerInitiateConnectionCommand::executeInternal() {
   socket->establishConnection(peer->ipaddr, peer->port);
   if(_mseHandshakeEnabled) {
     InitiatorMSEHandshakeCommand* c =
-      new InitiatorMSEHandshakeCommand(cuid, _requestGroup, peer, e, _btContext,
+      new InitiatorMSEHandshakeCommand(cuid, _requestGroup, peer, e,
 				       _btRuntime, socket);
     c->setPeerStorage(_peerStorage);
     c->setPieceStorage(_pieceStorage);
@@ -88,9 +89,8 @@ bool PeerInitiateConnectionCommand::executeInternal() {
   } else {
     PeerInteractionCommand* command =
       new PeerInteractionCommand
-      (cuid, _requestGroup, peer, e, _btContext, _btRuntime, _pieceStorage,
+      (cuid, _requestGroup, peer, e, _btRuntime, _pieceStorage, _peerStorage,
        socket, PeerInteractionCommand::INITIATOR_SEND_HANDSHAKE);
-    command->setPeerStorage(_peerStorage);
     e->commands.push_back(command);
   }
   return true;
@@ -103,7 +103,7 @@ bool PeerInitiateConnectionCommand::prepareForNextPeer(time_t wait) {
     peer->usedBy(e->newCUID());
     PeerInitiateConnectionCommand* command =
       new PeerInitiateConnectionCommand(peer->usedBy(), _requestGroup, peer, e,
-					_btContext, _btRuntime);
+					_btRuntime);
     command->setPeerStorage(_peerStorage);
     command->setPieceStorage(_pieceStorage);
     e->commands.push_back(command);
