@@ -36,18 +36,25 @@
 #define _D_PROGRESS_AWARE_ENTRY_H_
 
 #include "common.h"
+#include "SharedHandle.h"
+#include <stdint.h>
+#include <unistd.h>
+
+namespace aria2 {
 
 class ProgressAwareEntry {
 public:
   virtual ~ProgressAwareEntry() {}
 
-  virtual int64_t getCurrentLength() = 0;
+  virtual off_t getCurrentLength() = 0;
 
-  virtual int64_t getTotalLength() = 0;
+  virtual uint64_t getTotalLength() = 0;
 
   virtual bool finished() = 0;
 };
 
 typedef SharedHandle<ProgressAwareEntry> ProgressAwareEntryHandle;
 
+} // namespace aria2
+
 #endif // _D_PROGRESS_AWARE_ENTRY_H_
