@@ -40,29 +40,31 @@
 #include <deque>
 
 #include "SharedHandle.h"
+#include "PeerStat.h"
+#include "a2functional.h"
 
 namespace aria2 {
 
 class Request {
 private:
-  std::string url;
-  std::string currentUrl;
+  std::string _url;
+  std::string _currentUrl;
   /**
    * URL previously requested to the server. This is used as Referer
    */
-  std::string previousUrl;
+  std::string _previousUrl;
   /**
    * URL used as Referer in the initial request
    */
-  std::string referer;
-  std::string protocol;
-  std::string host;
-  uint16_t port;
-  std::string dir;
-  std::string file;
+  std::string _referer;
+  std::string _protocol;
+  std::string _host;
+  uint16_t _port;
+  std::string _dir;
+  std::string _file;
   /* after ? mark(includes '?' itself) */
   std::string _query;
-  unsigned int tryCount;
+  unsigned int _tryCount;
 
   unsigned int _redirectCount;
 
@@ -75,28 +77,32 @@ private:
   // maximum number of pipelined requests
   unsigned int _maxPipelinedRequest;
 
-  std::string method;
+  std::string _method;
 
   std::string _username;
 
   std::string _password;
 
+  bool _hasPassword;
+
+  bool _ipv6LiteralAddress;
+
+  SharedHandle<PeerStat> _peerStat;
+
   bool parseUrl(const std::string& url);
 public:
   Request();
 
-  // Parses URL and sets url, host, port, dir, file fields.
-  // Returns true if parsing goes successful, otherwise returns false.
+  // Sets url to _url and parses URL.  Returns true if parsing goes
+  // successful, otherwise returns false.
   bool setUrl(const std::string& url);
-  // Parses URL and sets host, port, dir, file fields.
-  // url field are not altered by this method.
-  // Returns true if parsing goes successful, otherwise returns false.
+  // Parses URL.  _url field are not altered by this method.  Returns
+  // true if parsing goes successful, otherwise returns false.
   bool redirectUrl(const std::string& url);
   bool resetUrl();
-  void resetTryCount() { tryCount = 0; }
-  void addTryCount() { ++tryCount; }
-  unsigned int getTryCount() const { return tryCount; }
-  //bool noMoreTry() const { return tryCount >= PREF_MAX_TRY; }
+  void resetTryCount() { _tryCount = 0; }
+  void addTryCount() { ++_tryCount; }
+  unsigned int getTryCount() const { return _tryCount; }
 
   void resetRedirectCount();
   
@@ -106,17 +112,28 @@ public:
   }
 
   // Returns URI passed by setUrl()
-  const std::string& getUrl() const { return url; }
-  const std::string& getCurrentUrl() const { return currentUrl; }
-  const std::string& getPreviousUrl() const { return previousUrl; }
-  const std::string& getReferer() const { return referer; }
+  const std::string& getUrl() const { return _url; }
+  const std::string& getCurrentUrl() const { return _currentUrl; }
+  const std::string& getPreviousUrl() const { return _previousUrl; }
+  const std::string& getReferer() const { return _referer; }
   void setReferer(const std::string& url);
-  const std::string& getProtocol() const { return protocol; }
-  const std::string& getHost() const { return host; }
-  uint16_t getPort() const { return port; }
-  const std::string& getDir() const { return dir; }
-  const std::string& getFile() const { return file;}
+  const std::string& getProtocol() const { return _protocol; }
+  const std::string& getHost() const { return _host; }
+  // Same as getHost(), but for IPv6 literal addresses, enclose them
+  // with square brackets and return.
+  std::string getURIHost() const
+  {
+    if(isIPv6LiteralAddress()) {
+      return strconcat("[", getHost(), "]");
+    } else {
+      return getHost();
+    }
+  }
+  uint16_t getPort() const { return _port; }
+  const std::string& getDir() const { return _dir; }
+  const std::string& getFile() const { return _file;}
   const std::string& getQuery() const { return _query; }
+  bool isIPv6LiteralAddress() const { return _ipv6LiteralAddress; }
 
   void supportsPersistentConnection(bool f)
   {
@@ -161,7 +178,7 @@ public:
   }
 
   void setMethod(const std::string& method) {
-    this->method = method;
+    _method = method;
   }
 
   const std::string& getUsername() const
@@ -174,10 +191,20 @@ public:
     return _password;
   }
 
-  const std::string& getMethod() const {
-    return method;
+  // Returns true if current URI has embedded password.
+  bool hasPassword() const
+  {
+    return _hasPassword;
   }
 
+  const std::string& getMethod() const {
+    return _method;
+  }
+
+  const SharedHandle<PeerStat>& getPeerStat() const { return _peerStat; }
+
+  const SharedHandle<PeerStat>& initPeerStat();
+
   static const std::string METHOD_GET;
   static const std::string METHOD_HEAD;
 
