@@ -44,11 +44,10 @@
 #include "NullProgressInfoFile.h"
 #include "Dependency.h"
 #include "prefs.h"
-#include "InitiateConnectionCommandFactory.h"
+#include "CreateRequestCommand.h"
 #include "File.h"
 #include "message.h"
-#include "Util.h"
-#include "BtRegistry.h"
+#include "util.h"
 #include "LogFactory.h"
 #include "Logger.h"
 #include "DiskAdaptor.h"
@@ -57,18 +56,15 @@
 #include "StreamCheckIntegrityEntry.h"
 #include "CheckIntegrityCommand.h"
 #include "UnknownLengthPieceStorage.h"
-#include "BtContext.h"
-#include "SingleFileDownloadContext.h"
+#include "DownloadContext.h"
 #include "DlAbortEx.h"
 #include "DownloadFailureException.h"
 #include "RequestGroupMan.h"
 #include "DefaultBtProgressInfoFile.h"
 #include "DefaultPieceStorage.h"
-#include "DownloadResult.h"
 #include "DownloadHandlerFactory.h"
 #include "MemoryBufferPreDownloadHandler.h"
 #include "DownloadHandlerConstants.h"
-#include "ServerHost.h"
 #include "Option.h"
 #include "FileEntry.h"
 #include "Request.h"
@@ -77,10 +73,16 @@
 #include "A2STR.h"
 #include "URISelector.h"
 #include "InOrderURISelector.h"
+#include "PieceSelector.h"
+#include "a2functional.h"
+#include "SocketCore.h"
+#include "SimpleRandomizer.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "CheckIntegrityCommand.h"
 #endif // ENABLE_MESSAGE_DIGEST
 #ifdef ENABLE_BITTORRENT
+# include "bittorrent_helper.h"
+# include "BtRegistry.h"
 # include "BtCheckIntegrityEntry.h"
 # include "DefaultPeerStorage.h"
 # include "DefaultBtAnnounce.h"
@@ -98,6 +100,8 @@
 # include "ExtensionMessageFactory.h"
 # include "DHTPeerAnnounceStorage.h"
 # include "DHTEntryPointNameResolveCommand.h"
+# include "LongestSequencePieceSelector.h"
+# include "PriorityPieceSelector.h"
 #endif // ENABLE_BITTORRENT
 #ifdef ENABLE_METALINK
 # include "MetalinkPostDownloadHandler.h"
@@ -109,44 +113,44 @@ int32_t RequestGroup::_gidCounter = 0;
 
 const std::string RequestGroup::ACCEPT_METALINK = "application/metalink+xml";
 
-RequestGroup::RequestGroup(const Option* option,
-			   const std::deque<std::string>& uris):
+RequestGroup::RequestGroup(const SharedHandle<Option>& option):
   _gid(++_gidCounter),
-  _uris(uris),
+  _option(new Option(*option.get())),
   _numConcurrentCommand(option->getAsInt(PREF_SPLIT)),
   _numStreamConnection(0),
   _numCommand(0),
-  _segmentManFactory(new DefaultSegmentManFactory(option)),
+  _segmentManFactory(new DefaultSegmentManFactory(_option.get())),
+  _saveControlFile(true),
   _progressInfoFile(new NullProgressInfoFile()),
   _preLocalFileCheckEnabled(true),
   _haltRequested(false),
   _forceHaltRequested(false),
-  _singleHostMultiConnectionEnabled(true),
+  _haltReason(RequestGroup::NONE),
   _uriSelector(new InOrderURISelector()),
   _lastModifiedTime(Time::null()),
   _fileNotFoundCount(0),
+  _timeout(option->getAsInt(PREF_TIMEOUT)),
   _inMemoryDownload(false),
-  _option(option),
+  _maxDownloadSpeedLimit(option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT)),
+  _maxUploadSpeedLimit(option->getAsInt(PREF_MAX_UPLOAD_LIMIT)),
+  _belongsToGID(0),
   _logger(LogFactory::getInstance())
 {
-  if(_option->get(PREF_FILE_ALLOCATION) == V_PREALLOC) {
-    _fileAllocationEnabled = true;
-  } else {
-    _fileAllocationEnabled = false;
-  }
+  _fileAllocationEnabled = _option->get(PREF_FILE_ALLOCATION) != V_NONE;
   // Add types to be sent as a Accept header value here.
   // It would be good to put this value in Option so that user can tweak
   // and add this list.
   // ACCEPT_METALINK is used for `transparent metalink'.
   addAcceptType(ACCEPT_METALINK);
-
-  initializePreDownloadHandler();
-  initializePostDownloadHandler();
+  if(!_option->getAsBool(PREF_DRY_RUN)) {
+    initializePreDownloadHandler();
+    initializePostDownloadHandler();
+  }
 }
 
 RequestGroup::~RequestGroup() {}
 
-SegmentManHandle RequestGroup::initSegmentMan()
+const SegmentManHandle& RequestGroup::initSegmentMan()
 {
   _segmentMan = _segmentManFactory->createNewInstance(_downloadContext,
 						      _pieceStorage);
@@ -171,6 +175,23 @@ bool RequestGroup::allDownloadFinished() const
   }
 }
 
+downloadresultcode::RESULT RequestGroup::downloadResult() const
+{
+  if (downloadFinished())
+    return downloadresultcode::FINISHED;
+  else {
+    if (_lastUriResult.isNull()) {
+      if(_haltReason == RequestGroup::USER_REQUEST) {
+	return downloadresultcode::IN_PROGRESS;
+      } else {
+	return downloadresultcode::UNKNOWN_ERROR;
+      }
+    } else {
+      return _lastUriResult->getResult();
+    }
+  }    
+}
+
 void RequestGroup::closeFile()
 {
   if(!_pieceStorage.isNull()) {
@@ -178,76 +199,122 @@ void RequestGroup::closeFile()
   }
 }
 
-void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
-					DownloadEngine* e,
-					const std::string& method)
+void RequestGroup::createInitialCommand
+(std::deque<Command*>& commands, DownloadEngine* e)
 {
 #ifdef ENABLE_BITTORRENT
   {
-    BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
-    if(!btContext.isNull()) {
-      if(e->_requestGroupMan->isSameFileBeingDownloaded(this)) {
-	throw DownloadFailureException
-	  (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-			getFilePath().c_str()).str());
+    if(_downloadContext->hasAttribute(bittorrent::BITTORRENT)) {
+      const BDE& torrentAttrs =
+	_downloadContext->getAttribute(bittorrent::BITTORRENT);
+      bool metadataGetMode = !torrentAttrs.containsKey(bittorrent::METADATA);
+      if(_option->getAsBool(PREF_DRY_RUN)) {
+	throw DOWNLOAD_FAILURE_EXCEPTION
+	  ("Cancel BitTorrent download in dry-run context.");
       }
-      initPieceStorage();
-      if(btContext->getFileEntries().size() > 1) {
-	// this is really multi file torrent.
-	// clear http/ftp uris because the current implementation does not
-	// allow integrating multi-file torrent and http/ftp.
-	_logger->debug("Clearing http/ftp URIs because the current implementation does not allow integrating multi-file torrent and http/ftp.");
-	_uris.clear();
-
-	_pieceStorage->setFileFilter(Util::parseIntRange(_option->get(PREF_SELECT_FILE)));
-      }
-      
-      SharedHandle<DefaultBtProgressInfoFile>
-	progressInfoFile(new DefaultBtProgressInfoFile(_downloadContext,
-						       _pieceStorage,
-						       _option));
-      
       SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
-
-      btRegistry->registerBtContext(btContext->getInfoHashAsString(),
-				    btContext);
-      btRegistry->registerPieceStorage(btContext->getInfoHashAsString(),
-				       _pieceStorage);
-      btRegistry->registerBtProgressInfoFile(btContext->getInfoHashAsString(),
-					     progressInfoFile);
-
-  
+      if(!btRegistry->getDownloadContext
+	 (torrentAttrs[bittorrent::INFO_HASH].s()).isNull()) {
+	throw DOWNLOAD_FAILURE_EXCEPTION
+	  (StringFormat
+	   ("InfoHash %s is already registered.",
+	    bittorrent::getInfoHashString(_downloadContext).c_str()).str());
+      }
+      if(metadataGetMode) {
+	// Use UnknownLengthPieceStorage.
+	initPieceStorage();
+      } else {
+	if(e->_requestGroupMan->isSameFileBeingDownloaded(this)) {
+	  throw DOWNLOAD_FAILURE_EXCEPTION
+	    (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
+			  _downloadContext->getBasePath().c_str()).str());
+	}
+	initPieceStorage();
+	if(_downloadContext->getFileEntries().size() > 1) {
+	  _pieceStorage->setupFileFilter();
+	}
+      }
+      
+      SharedHandle<DefaultBtProgressInfoFile> progressInfoFile;
+      if(!metadataGetMode) {
+	progressInfoFile.reset(new DefaultBtProgressInfoFile(_downloadContext,
+							     _pieceStorage,
+							     _option.get()));
+      }
+        
       BtRuntimeHandle btRuntime(new BtRuntime());
-      btRuntime->setListenPort(_option->getAsInt(PREF_LISTEN_PORT));
-      btRegistry->registerBtRuntime(btContext->getInfoHashAsString(),
-				    btRuntime);
+      btRuntime->setMaxPeers(_option->getAsInt(PREF_BT_MAX_PEERS));
       _btRuntime = btRuntime;
-      progressInfoFile->setBtRuntime(btRuntime);
+      if(!progressInfoFile.isNull()) {
+	progressInfoFile->setBtRuntime(btRuntime);
+      }
 
       SharedHandle<DefaultPeerStorage> peerStorage
-	(new DefaultPeerStorage(btContext, _option));
+	(new DefaultPeerStorage(_option.get()));
       peerStorage->setBtRuntime(btRuntime);
       peerStorage->setPieceStorage(_pieceStorage);
-      btRegistry->registerPeerStorage(btContext->getInfoHashAsString(),
-				      peerStorage);
       _peerStorage = peerStorage;
-      progressInfoFile->setPeerStorage(peerStorage);
+      if(!progressInfoFile.isNull()) {
+	progressInfoFile->setPeerStorage(peerStorage);
+      }
 
       SharedHandle<DefaultBtAnnounce> btAnnounce
-	(new DefaultBtAnnounce(btContext, _option));
+	(new DefaultBtAnnounce(_downloadContext, _option.get()));
       btAnnounce->setBtRuntime(btRuntime);
       btAnnounce->setPieceStorage(_pieceStorage);
       btAnnounce->setPeerStorage(peerStorage);
-      btRegistry->registerBtAnnounce(btContext->getInfoHashAsString(),
-				     btAnnounce);
+      btAnnounce->setUserDefinedInterval
+	(_option->getAsInt(PREF_BT_TRACKER_INTERVAL));
       btAnnounce->shuffleAnnounce();
       
+      btRegistry->put(_gid,
+		      BtObject(_downloadContext,
+			       _pieceStorage,
+			       peerStorage,
+			       btAnnounce,
+			       btRuntime,
+			       (progressInfoFile.isNull()?
+				_progressInfoFile:
+				SharedHandle<BtProgressInfoFile>
+				(progressInfoFile))));
+      if(metadataGetMode) {
+	if(_option->getAsBool(PREF_ENABLE_DHT)) {
+	  std::deque<Command*> dhtCommands;
+	  DHTSetup().setup(dhtCommands, e, _option.get());
+	  e->addCommand(dhtCommands);
+	} else {
+	  _logger->notice("For BitTorrent Magnet URI, enabling DHT is strongly"
+			  " recommended. See --enable-dht option.");
+	}
+
+	SharedHandle<CheckIntegrityEntry> entry
+	  (new BtCheckIntegrityEntry(this));
+	entry->onDownloadIncomplete(commands, e);
+	
+	return;
+      }
+
       // Remove the control file if download file doesn't exist
       if(progressInfoFile->exists() && !_pieceStorage->getDiskAdaptor()->fileExists()) {
 	progressInfoFile->removeFile();
 	_logger->notice(MSG_REMOVED_DEFUNCT_CONTROL_FILE,
 			progressInfoFile->getFilename().c_str(),
-			_pieceStorage->getDiskAdaptor()->getFilePath().c_str());
+			_downloadContext->getBasePath().c_str());
+      }
+      {
+	uint64_t actualFileSize = _pieceStorage->getDiskAdaptor()->size();
+	if(actualFileSize == _downloadContext->getTotalLength()) {
+	  // First, make DiskAdaptor read-only mode to allow the
+	  // program to seed file in read-only media.
+	  _pieceStorage->getDiskAdaptor()->enableReadOnly();
+	} else {
+	  // Open file in writable mode to allow the program
+	  // truncate the file to _downloadContext->getTotalLength()
+	  _logger->debug("File size not match. File is opened in writable mode."
+			 " Expected:%s Actual:%s",
+			 util::uitos(_downloadContext->getTotalLength()).c_str(),
+			 util::uitos(actualFileSize).c_str());
+	}
       }
       // Call Load, Save and file allocation command here
       if(progressInfoFile->exists()) {
@@ -260,10 +327,10 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 	     !_option->getAsBool(PREF_ALLOW_OVERWRITE) &&
 	     !_option->getAsBool(PREF_BT_SEED_UNVERIFIED)) {
 	    // TODO we need this->haltRequested = true?
-	    throw DownloadFailureException
+	    throw DOWNLOAD_FAILURE_EXCEPTION
 	      (StringFormat
 	       (MSG_FILE_ALREADY_EXISTS,
-		_pieceStorage->getDiskAdaptor()->getFilePath().c_str()).str());
+		_downloadContext->getBasePath().c_str()).str());
 	  } else {
 	    _pieceStorage->getDiskAdaptor()->openFile();
 	  }
@@ -276,14 +343,22 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
       }
       _progressInfoFile = progressInfoFile;
 
-      if(!btContext->isPrivate() && _option->getAsBool(PREF_ENABLE_DHT)) {
+      if(torrentAttrs[bittorrent::PRIVATE].i() == 0 &&
+	 _option->getAsBool(PREF_ENABLE_DHT)) {
 	std::deque<Command*> commands;
-	DHTSetup().setup(commands, e, _option);
+	DHTSetup().setup(commands, e, _option.get());
 	e->addCommand(commands);
-	if(!btContext->getNodes().empty() && DHTSetup::initialized()) {
+	if(!torrentAttrs[bittorrent::NODES].empty() && DHTSetup::initialized()) {
+	  std::deque<std::pair<std::string, uint16_t> > entryPoints;
+	  const BDE& nodes = torrentAttrs[bittorrent::NODES];
+	  for(BDE::List::const_iterator i = nodes.listBegin();
+	      i != nodes.listEnd(); ++i) {
+	    std::pair<std::string, uint16_t> addr
+	      ((*i)[bittorrent::HOSTNAME].s(), (*i)[bittorrent::PORT].i());
+	    entryPoints.push_back(addr);
+	  }
 	  DHTEntryPointNameResolveCommand* command =
-	    new DHTEntryPointNameResolveCommand(e->newCUID(), e,
-						btContext->getNodes());
+	    new DHTEntryPointNameResolveCommand(e->newCUID(), e, entryPoints);
 	  command->setTaskQueue(DHTRegistry::_taskQueue);
 	  command->setTaskFactory(DHTRegistry::_taskFactory);
 	  command->setRoutingTable(DHTRegistry::_routingTable);
@@ -304,23 +379,40 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
     }
   }
 #endif // ENABLE_BITTORRENT
+  // TODO Currently, BitTorrent+WEB-Seeding is only way to download
+  // multiple files in one RequestGroup. In this context, we don't
+  // have BitTorrent, so add assertion here. This situation will be
+  // changed if Metalink spec is formalized to support multi-file
+  // torrent.
+  assert(_downloadContext->getFileEntries().size() == 1);
   // TODO I assume here when totallength is set to DownloadContext and it is
   // not 0, then filepath is also set DownloadContext correctly....
-  if(_downloadContext->getTotalLength() == 0) {
-    createNextCommand(commands, e, 1, method);
+  if(_option->getAsBool(PREF_DRY_RUN) ||
+     _downloadContext->getTotalLength() == 0) {
+    createNextCommand(commands, e, 1);
   }else {
     if(e->_requestGroupMan->isSameFileBeingDownloaded(this)) {
-      throw DownloadFailureException
+      throw DOWNLOAD_FAILURE_EXCEPTION
 	(StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-		      getFilePath().c_str()).str());
+		      _downloadContext->getBasePath().c_str()).str());
     }
+    adjustFilename
+      (SharedHandle<BtProgressInfoFile>(new DefaultBtProgressInfoFile
+					(_downloadContext,
+					 SharedHandle<PieceStorage>(),
+					 _option.get())));
     initPieceStorage();
     BtProgressInfoFileHandle infoFile
-      (new DefaultBtProgressInfoFile(_downloadContext, _pieceStorage, _option));
-    if(infoFile->exists() || !downloadFinishedByFileLength()) {
+      (new DefaultBtProgressInfoFile(_downloadContext, _pieceStorage,
+				     _option.get()));
+    if(!infoFile->exists() && downloadFinishedByFileLength()) {
+      _pieceStorage->markAllPiecesDone();
+      _logger->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
+		      _gid, _downloadContext->getBasePath().c_str());
+    } else {
       loadAndOpenFile(infoFile);
       SharedHandle<CheckIntegrityEntry> checkIntegrityEntry
-	(new StreamCheckIntegrityEntry(SharedHandle<Request>(), this));
+	(new StreamCheckIntegrityEntry(this));
       processCheckIntegrityEntry(commands, checkIntegrityEntry, e);
     }
   }
@@ -331,13 +423,18 @@ void RequestGroup::processCheckIntegrityEntry(std::deque<Command*>& commands,
 					      DownloadEngine* e)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
-  if(e->option->getAsBool(PREF_CHECK_INTEGRITY) &&
+  if(_option->getAsBool(PREF_CHECK_INTEGRITY) &&
      entry->isValidationReady()) {
     entry->initValidator();
     entry->cutTrailingGarbage();
-    CheckIntegrityCommand* command =
-      new CheckIntegrityCommand(e->newCUID(), this, e, entry);
-    commands.push_back(command);
+    // Don't save control file(.aria2 file) when user presses
+    // control-c key while aria2 is checking hashes. If control file
+    // doesn't exist when aria2 launched, the completed length in
+    // saved control file will be 0 byte and this confuses user.
+    // enableSaveControlFile() will be called after hash checking is
+    // done. See CheckIntegrityCommand.
+    disableSaveControlFile();
+    e->_checkIntegrityMan->pushEntry(entry);
   } else
 #endif // ENABLE_MESSAGE_DIGEST
     {
@@ -348,15 +445,47 @@ void RequestGroup::processCheckIntegrityEntry(std::deque<Command*>& commands,
 void RequestGroup::initPieceStorage()
 {
   if(_downloadContext->knowsTotalLength()) {
-    DefaultPieceStorageHandle ps
-      (new DefaultPieceStorage(_downloadContext, _option));
+#ifdef ENABLE_BITTORRENT
+    SharedHandle<DefaultPieceStorage> ps
+      (new DefaultPieceStorage(_downloadContext, _option.get()));
+    if(_downloadContext->hasAttribute(bittorrent::BITTORRENT)) {
+      if(isUriSuppliedForRequsetFileEntry
+	 (_downloadContext->getFileEntries().begin(),
+	  _downloadContext->getFileEntries().end())) {
+	// Use LongestSequencePieceSelector when HTTP/FTP/BitTorrent
+	// integrated downloads. Currently multi-file integrated
+	// download is not supported.
+	_logger->debug("Using LongestSequencePieceSelector");
+	ps->setPieceSelector
+	  (SharedHandle<PieceSelector>(new LongestSequencePieceSelector()));
+      }
+      if(_option->defined(PREF_BT_PRIORITIZE_PIECE)) {
+	std::vector<size_t> result;
+	util::parsePrioritizePieceRange
+	  (result, _option->get(PREF_BT_PRIORITIZE_PIECE),
+	   _downloadContext->getFileEntries(),
+	   _downloadContext->getPieceLength());
+	if(!result.empty()) {
+	  std::random_shuffle(result.begin(), result.end(),
+			      *(SimpleRandomizer::getInstance().get()));
+	  SharedHandle<PriorityPieceSelector> priSelector
+	    (new PriorityPieceSelector(ps->getPieceSelector()));
+	  priSelector->setPriorityPiece(result.begin(), result.end());
+	  ps->setPieceSelector(priSelector);
+	}
+      }
+    }
+#else // !ENABLE_BITTORRENT
+    SharedHandle<DefaultPieceStorage> ps
+      (new DefaultPieceStorage(_downloadContext, _option.get()));
+#endif // !ENABLE_BITTORRENT
     if(!_diskWriterFactory.isNull()) {
       ps->setDiskWriterFactory(_diskWriterFactory);
     }
     _pieceStorage = ps;
   } else {
     UnknownLengthPieceStorageHandle ps
-      (new UnknownLengthPieceStorage(_downloadContext, _option));
+      (new UnknownLengthPieceStorage(_downloadContext, _option.get()));
     if(!_diskWriterFactory.isNull()) {
       ps->setDiskWriterFactory(_diskWriterFactory);
     }
@@ -375,17 +504,45 @@ bool RequestGroup::downloadFinishedByFileLength()
       !_downloadContext->getPieceHashes().empty())) {
     return false;
   }
-  // TODO consider the case when the getFilePath() returns dir path. 
-  File outfile(getFilePath());
-  if(outfile.exists() && getTotalLength() == outfile.size()) {
-    _pieceStorage->markAllPiecesDone();
-    _logger->notice(MSG_DOWNLOAD_ALREADY_COMPLETED, _gid, getFilePath().c_str());
+  if(!_downloadContext->knowsTotalLength()) {
+    return false;
+  }
+  File outfile(getFirstFilePath());
+  if(outfile.exists() && _downloadContext->getTotalLength() == outfile.size()) {
     return true;
   } else {
     return false;
   }
 }
 
+void RequestGroup::adjustFilename
+(const SharedHandle<BtProgressInfoFile>& infoFile)
+{
+  if(!isPreLocalFileCheckEnabled()) {
+    // OK, no need to care about filename.
+  } else if(infoFile->exists()) {
+    // Use current filename
+  } else if(downloadFinishedByFileLength()) {
+    // File was downloaded already, no need to change file name.
+  } else {
+    File outfile(getFirstFilePath());    
+    if(outfile.exists() && _option->getAsBool(PREF_CONTINUE) &&
+       outfile.size() <= _downloadContext->getTotalLength()) {
+      // File exists but user decided to resume it.
+    } else {
+#ifdef ENABLE_MESSAGE_DIGEST
+      if(outfile.exists() && _option->getAsBool(PREF_CHECK_INTEGRITY)) {
+	// check-integrity existing file
+      } else {
+#endif // ENABLE_MESSAGE_DIGEST
+	shouldCancelDownloadForSafety();
+#ifdef ENABLE_MESSAGE_DIGEST
+      }
+#endif // ENABLE_MESSAGE_DIGEST
+    }
+  }
+}
+
 void RequestGroup::loadAndOpenFile(const BtProgressInfoFileHandle& progressInfoFile)
 {
   try {
@@ -398,45 +555,33 @@ void RequestGroup::loadAndOpenFile(const BtProgressInfoFileHandle& progressInfoF
       progressInfoFile->removeFile();
       _logger->notice(MSG_REMOVED_DEFUNCT_CONTROL_FILE,
 		      progressInfoFile->getFilename().c_str(),
-		      _pieceStorage->getDiskAdaptor()->getFilePath().c_str());
+		      _downloadContext->getBasePath().c_str());
     }
-    while(1) {
-      if(progressInfoFile->exists()) {
-	progressInfoFile->load();
+
+    if(progressInfoFile->exists()) {
+      progressInfoFile->load();
+      _pieceStorage->getDiskAdaptor()->openExistingFile();
+    } else {
+      File outfile(getFirstFilePath());    
+      if(outfile.exists() && _option->getAsBool(PREF_CONTINUE) &&
+	 outfile.size() <= getTotalLength()) {
 	_pieceStorage->getDiskAdaptor()->openExistingFile();
+	_pieceStorage->markPiecesDone(outfile.size());
       } else {
-	File outfile(getFilePath());    
-	if(outfile.exists() && _option->getAsBool(PREF_CONTINUE) &&
-	   outfile.size() <= getTotalLength()) {
+#ifdef ENABLE_MESSAGE_DIGEST
+	if(outfile.exists() && _option->getAsBool(PREF_CHECK_INTEGRITY)) {
 	  _pieceStorage->getDiskAdaptor()->openExistingFile();
-	  _pieceStorage->markPiecesDone(outfile.size());
 	} else {
-#ifdef ENABLE_MESSAGE_DIGEST
-	  if(outfile.exists() && _option->getAsBool(PREF_CHECK_INTEGRITY)) {
-	    _pieceStorage->getDiskAdaptor()->openExistingFile();
-	  } else {
 #endif // ENABLE_MESSAGE_DIGEST
-	    shouldCancelDownloadForSafety();
-	    // call updateFilename here in case when filename is renamed
-	    // by tryAutoFileRenaming()
-	    progressInfoFile->updateFilename();
-	    if(progressInfoFile->exists()) {
-	      // Close DiskAdaptor here. Renmaed file will be opened in the
-	      // next loop .
-	      _pieceStorage->getDiskAdaptor()->closeFile();
-	      continue;
-	    }
-	    _pieceStorage->getDiskAdaptor()->initAndOpenFile();
+	  _pieceStorage->getDiskAdaptor()->initAndOpenFile();
 #ifdef ENABLE_MESSAGE_DIGEST
-	  }
-#endif // ENABLE_MESSAGE_DIGEST
 	}
+#endif // ENABLE_MESSAGE_DIGEST
       }
-      setProgressInfoFile(progressInfoFile);
-      break;
     }
+    setProgressInfoFile(progressInfoFile);
   } catch(RecoverableException& e) {
-    throw DownloadFailureException
+    throw DOWNLOAD_FAILURE_EXCEPTION2
       (StringFormat(EX_DOWNLOAD_ABORTED).str(), e);
   }
 }
@@ -447,44 +592,35 @@ void RequestGroup::shouldCancelDownloadForSafety()
   if(_option->getAsBool(PREF_ALLOW_OVERWRITE)) {
     return;
   }
-  File outfile(getFilePath());
+  File outfile(getFirstFilePath());
   if(outfile.exists()) {
     if(_option->getAsBool(PREF_AUTO_FILE_RENAMING)) {
       if(tryAutoFileRenaming()) {
-	_logger->notice(MSG_FILE_RENAMED, getFilePath().c_str());
+	_logger->notice(MSG_FILE_RENAMED, getFirstFilePath().c_str());
       } else {
-	throw DownloadFailureException
+	throw DOWNLOAD_FAILURE_EXCEPTION
 	  (StringFormat("File renaming failed: %s",
-			getFilePath().c_str()).str());
+			getFirstFilePath().c_str()).str());
       }
     } else {
-      throw DownloadFailureException
+      throw DOWNLOAD_FAILURE_EXCEPTION
 	(StringFormat(MSG_FILE_ALREADY_EXISTS,
-		      getFilePath().c_str()).str());
+		      getFirstFilePath().c_str()).str());
     }
   }
 }
 
 bool RequestGroup::tryAutoFileRenaming()
 {
-  std::string filepath = getFilePath();
+  std::string filepath = getFirstFilePath();
   if(filepath.empty()) {
     return false;
   }
-  SingleFileDownloadContextHandle ctx =
-    dynamic_pointer_cast<SingleFileDownloadContext>(_downloadContext);
-  // Make a copy of ctx.
-  SingleFileDownloadContextHandle tempCtx(new SingleFileDownloadContext(*ctx.get()));
-
-  DefaultBtProgressInfoFile tempInfoFile(tempCtx, SharedHandle<PieceStorage>(), 0);
-
   for(unsigned int i = 1; i < 10000; ++i) {
-    File newfile(filepath+"."+Util::uitos(i));
-    std::string newFilename = newfile.getBasename();
-    tempCtx->setUFilename(newFilename);
-    tempInfoFile.updateFilename();
-    if(!newfile.exists() || (newfile.exists() && tempInfoFile.exists())) {
-      ctx->setUFilename(newFilename);
+    File newfile(strconcat(filepath, ".", util::uitos(i)));
+    File ctrlfile(newfile.getPath()+DefaultBtProgressInfoFile::getSuffix());
+    if(!newfile.exists() || (newfile.exists() && ctrlfile.exists())) {
+      _downloadContext->getFirstFileEntry()->setPath(newfile.getPath());
       return true;
     }
   }
@@ -498,14 +634,8 @@ void RequestGroup::createNextCommandWithAdj(std::deque<Command*>& commands,
   if(getTotalLength() == 0) {
     numCommand = 1+numAdj;
   } else {
-    if(_numConcurrentCommand == 0) {
-      // TODO remove _uris.size() support
-      numCommand = _uris.size();
-    } else {
-      numCommand = _numConcurrentCommand;
-    }
-    numCommand = std::min(static_cast<int>(_downloadContext->getNumPieces()),
-			  numCommand);
+    numCommand = std::min(_downloadContext->getNumPieces(),
+			  _numConcurrentCommand);
     numCommand += numAdj;
   }
   if(numCommand > 0) {
@@ -515,49 +645,25 @@ void RequestGroup::createNextCommandWithAdj(std::deque<Command*>& commands,
 
 void RequestGroup::createNextCommand(std::deque<Command*>& commands,
 				     DownloadEngine* e,
-				     unsigned int numCommand,
-				     const std::string& method)
+				     unsigned int numCommand)
 {
-  std::deque<std::string> pendingURIs;
-  for(; !_uris.empty() && numCommand--; ) {    
-    std::string uri = _uriSelector->select(_uris);
-    RequestHandle req(new Request());
-    if(req->setUrl(uri)) {
-      ServerHostHandle sv;
-      if(!_singleHostMultiConnectionEnabled){
-	sv = searchServerHost(req->getHost());
-      }
-      if(sv.isNull()) {
-	_spentUris.push_back(uri);
-	req->setReferer(_option->get(PREF_REFERER));
-	req->setMethod(method);
-
-	Command* command =
-	  InitiateConnectionCommandFactory::createInitiateConnectionCommand
-	  (e->newCUID(), req, this, e);
-	ServerHostHandle sv(new ServerHost(command->getCuid(), req->getHost()));
-	registerServerHost(sv);
-	// give a chance to be executed in the next loop in DownloadEngine
-	command->setStatus(Command::STATUS_ONESHOT_REALTIME);
-	commands.push_back(command);
-      } else {
-	pendingURIs.push_back(uri);
-      }
-    } else {
-      _logger->error(MSG_UNRECOGNIZED_URI, req->getUrl().c_str());
-    }
+  for(; numCommand--; ) {
+    Command* command = new CreateRequestCommand(e->newCUID(), this, e);
+    commands.push_back(command);
+  }
+  if(!commands.empty()) {
+    e->setNoWait(true);
   }
-  _uris.insert(_uris.begin(), pendingURIs.begin(), pendingURIs.end());
 }
 
-std::string RequestGroup::getFilePath() const
+std::string RequestGroup::getFirstFilePath() const
 {
   assert(!_downloadContext.isNull());
   if(inMemoryDownload()) {
     static const std::string DIR_MEMORY("[MEMORY]");
-    return DIR_MEMORY+File(_downloadContext->getActualBasePath()).getBasename();
+    return DIR_MEMORY+File(_downloadContext->getFirstFileEntry()->getPath()).getBasename();
   } else {
-    return _downloadContext->getActualBasePath();
+    return _downloadContext->getFirstFileEntry()->getPath();
   }
 }
 
@@ -594,7 +700,7 @@ void RequestGroup::validateFilename(const std::string& expectedFilename,
     return;
   }
   if(expectedFilename != actualFilename) {
-    throw DlAbortEx(StringFormat(EX_FILENAME_MISMATCH,
+    throw DL_ABORT_EX(StringFormat(EX_FILENAME_MISMATCH,
 				 expectedFilename.c_str(),
 				 actualFilename.c_str()).str());
   }
@@ -607,10 +713,10 @@ void RequestGroup::validateTotalLength(uint64_t expectedTotalLength,
     return;
   }
   if(expectedTotalLength != actualTotalLength) {
-    throw DlAbortEx
+    throw DL_ABORT_EX
       (StringFormat(EX_SIZE_MISMATCH,
-		    Util::itos(expectedTotalLength, true).c_str(),
-		    Util::itos(actualTotalLength, true).c_str()).str());
+		    util::itos(expectedTotalLength, true).c_str(),
+		    util::itos(actualTotalLength, true).c_str()).str());
   }
 }
 
@@ -670,9 +776,12 @@ TransferStat RequestGroup::calculateStat()
   return stat;
 }
 
-void RequestGroup::setHaltRequested(bool f)
+void RequestGroup::setHaltRequested(bool f, HaltReason haltReason)
 {
   _haltRequested = f;
+  if(_haltRequested) {
+    _haltReason = haltReason;
+  }
 #ifdef ENABLE_BITTORRENT
   if(!_btRuntime.isNull()) {
     _btRuntime->setHalt(f);
@@ -680,39 +789,27 @@ void RequestGroup::setHaltRequested(bool f)
 #endif // ENABLE_BITTORRENT
 }
 
-void RequestGroup::setForceHaltRequested(bool f)
+void RequestGroup::setForceHaltRequested(bool f, HaltReason haltReason)
 {
-  setHaltRequested(f);
+  setHaltRequested(f, haltReason);
   _forceHaltRequested = f;
 }
 
 void RequestGroup::releaseRuntimeResource(DownloadEngine* e)
 {
 #ifdef ENABLE_BITTORRENT
-  BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
-  if(!btContext.isNull()) {
-    SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
-    BtContextHandle btContextInReg =
-      btRegistry->getBtContext(btContext->getInfoHashAsString());
-    if(!btContextInReg.isNull() &&
-       btContextInReg->getOwnerRequestGroup()->getGID() ==
-	btContext->getOwnerRequestGroup()->getGID()) {
-      btRegistry->unregister(btContext->getInfoHashAsString());
-      if(!DHTRegistry::_peerAnnounceStorage.isNull()) {
-	DHTRegistry::_peerAnnounceStorage->
-	  removeLocalPeerAnnounce(btContext->getInfoHash());
-      }
-    }
-  }
+  e->getBtRegistry()->remove(_gid);
 #endif // ENABLE_BITTORRENT
   if(!_pieceStorage.isNull()) {
     _pieceStorage->removeAdvertisedPiece(0);
   }
+  _downloadContext->releaseRuntimeResource();
 }
 
 void RequestGroup::preDownloadProcessing()
 {
-  _logger->debug("Finding PreDownloadHandler for path %s.", getFilePath().c_str());
+  _logger->debug("Finding PreDownloadHandler for path %s.",
+		 getFirstFilePath().c_str());
   try {
     for(PreDownloadHandlers::const_iterator itr = _preDownloadHandlers.begin();
 	itr != _preDownloadHandlers.end(); ++itr) {
@@ -732,7 +829,8 @@ void RequestGroup::preDownloadProcessing()
 void RequestGroup::postDownloadProcessing
 (std::deque<SharedHandle<RequestGroup> >& groups)
 {
-  _logger->debug("Finding PostDownloadHandler for path %s.", getFilePath().c_str());
+  _logger->debug("Finding PostDownloadHandler for path %s.",
+		 getFirstFilePath().c_str());
   try {
     for(PostDownloadHandlers::const_iterator itr = _postDownloadHandlers.begin();
 	itr != _postDownloadHandlers.end(); ++itr) {
@@ -777,12 +875,6 @@ void RequestGroup::initializePostDownloadHandler()
 #endif // ENABLE_METALINK
 }
 
-void RequestGroup::getURIs(std::deque<std::string>& uris) const
-{
-  uris.insert(uris.end(), _spentUris.begin(), _spentUris.end());
-  uris.insert(uris.end(), _uris.begin(), _uris.end());
-}
-
 bool RequestGroup::isDependencyResolved()
 {
   if(_dependency.isNull()) {
@@ -806,11 +898,6 @@ void RequestGroup::setDiskWriterFactory(const DiskWriterFactoryHandle& diskWrite
   _diskWriterFactory = diskWriterFactory;
 }
 
-DiskWriterFactoryHandle RequestGroup::getDiskWriterFactory() const
-{
-  return _diskWriterFactory;
-}
-
 void RequestGroup::addPostDownloadHandler(const PostDownloadHandlerHandle& handler)
 {
   _postDownloadHandlers.push_back(handler);
@@ -821,46 +908,21 @@ void RequestGroup::addPreDownloadHandler(const PreDownloadHandlerHandle& handler
   _preDownloadHandlers.push_back(handler);
 }
 
-void RequestGroup::clearPostDowloadHandler()
+void RequestGroup::clearPostDownloadHandler()
 {
   _postDownloadHandlers.clear();
 }
 
-void RequestGroup::clearPreDowloadHandler()
+void RequestGroup::clearPreDownloadHandler()
 {
   _preDownloadHandlers.clear();
 }
 
-SegmentManHandle RequestGroup::getSegmentMan() const
-{
-  return _segmentMan;
-}
-
-DownloadContextHandle RequestGroup::getDownloadContext() const
-{
-  return _downloadContext;
-}
-
-void RequestGroup::setDownloadContext(const DownloadContextHandle& downloadContext)
-{
-  _downloadContext = downloadContext;
-}
-
-PieceStorageHandle RequestGroup::getPieceStorage() const
-{
-  return _pieceStorage;
-}
-
 void RequestGroup::setPieceStorage(const PieceStorageHandle& pieceStorage)
 {
   _pieceStorage = pieceStorage;
 }
 
-BtProgressInfoFileHandle RequestGroup::getProgressInfoFile() const
-{
-  return _progressInfoFile;
-}
-
 void RequestGroup::setProgressInfoFile(const BtProgressInfoFileHandle& progressInfoFile)
 {
   _progressInfoFile = progressInfoFile;
@@ -875,156 +937,51 @@ bool RequestGroup::needsFileAllocation() const
 
 DownloadResultHandle RequestGroup::createDownloadResult() const
 {
-  std::deque<std::string> uris;
-  getURIs(uris);
-
   uint64_t sessionDownloadLength = 0;
 
 #ifdef ENABLE_BITTORRENT
   if(!_peerStorage.isNull()) {
-    sessionDownloadLength =
+    sessionDownloadLength +=
       _peerStorage->calculateStat().getSessionDownloadLength();
-  } else
+  }
 #endif // ENABLE_BITTORRENT
-    if(!_segmentMan.isNull()) {
-      sessionDownloadLength =
-	_segmentMan->calculateSessionDownloadLength();
-    }
-
+  if(!_segmentMan.isNull()) {
+    sessionDownloadLength +=
+      _segmentMan->calculateSessionDownloadLength();
+  }
   return
     SharedHandle<DownloadResult>
     (new DownloadResult(_gid,
-			getFilePath(),
-			getTotalLength(),
-			uris.empty() ? A2STR::NIL:uris.front(),
-			uris.size(),
+			_downloadContext->getFileEntries(),
+			_inMemoryDownload,
 			sessionDownloadLength,
 			_downloadContext->calculateSessionTime(),
-			downloadFinished()?
-			DownloadResult::FINISHED :
-			DownloadResult::NOT_YET));
-}
-
-void RequestGroup::registerServerHost(const ServerHostHandle& serverHost)
-{
-  _serverHosts.push_back(serverHost);
-}
-
-class FindServerHostByCUID
-{
-private:
-  int32_t _cuid;
-public:
-  FindServerHostByCUID(int32_t cuid):_cuid(cuid) {}
-
-  bool operator()(const ServerHostHandle& sv) const
-  {
-    return sv->getCuid() == _cuid;
-  }
-};
-
-ServerHostHandle RequestGroup::searchServerHost(int32_t cuid) const
-{
-  std::deque<SharedHandle<ServerHost> >::const_iterator itr =
-    std::find_if(_serverHosts.begin(), _serverHosts.end(), FindServerHostByCUID(cuid));
-  if(itr == _serverHosts.end()) {
-    return SharedHandle<ServerHost>();
-  } else {
-    return *itr;
-  }
-}
-
-class FindServerHostByHostname
-{
-private:
-  std::string _hostname;
-public:
-  FindServerHostByHostname(const std::string& hostname):_hostname(hostname) {}
-
-  bool operator()(const ServerHostHandle& sv) const
-  {
-    return sv->getHostname() == _hostname;
-  }
-};
-
-ServerHostHandle RequestGroup::searchServerHost(const std::string& hostname) const
-{
-  std::deque<SharedHandle<ServerHost> >::const_iterator itr =
-    std::find_if(_serverHosts.begin(), _serverHosts.end(), FindServerHostByHostname(hostname));
-  if(itr == _serverHosts.end()) {
-    return SharedHandle<ServerHost>();
-  } else {
-    return *itr;
-  }
-}
-
-void RequestGroup::removeServerHost(int32_t cuid)
-{
-  _serverHosts.erase(std::remove_if(_serverHosts.begin(), _serverHosts.end(), FindServerHostByCUID(cuid)), _serverHosts.end());
+			downloadResult(),
+			_followedByGIDs,
+			_belongsToGID));
 }
   
-void RequestGroup::removeURIWhoseHostnameIs(const std::string& hostname)
-{
-  std::deque<std::string> newURIs;
-  Request req;
-  for(std::deque<std::string>::const_iterator itr = _uris.begin(); itr != _uris.end(); ++itr) {
-    if(((*itr).find(hostname) == std::string::npos) ||
-       (req.setUrl(*itr) && (req.getHost() != hostname))) {
-      newURIs.push_back(*itr);
-    }
-  }
-  _logger->debug("GUID#%d - Removed %d duplicate hostname URIs",
-		 _gid, _uris.size()-newURIs.size());
-  _uris = newURIs;
-}
-
-void RequestGroup::removeIdenticalURI(const std::string& uri)
-{
-  _uris.erase(std::remove(_uris.begin(), _uris.end(), uri), _uris.end());
-}
-
 void RequestGroup::reportDownloadFinished()
 {
   _logger->notice(MSG_FILE_DOWNLOAD_COMPLETED,
-		  getFilePath().c_str());
+		  _downloadContext->getBasePath().c_str());
+  _uriSelector->resetCounters();
 #ifdef ENABLE_BITTORRENT
-  SharedHandle<BtContext> ctx = dynamic_pointer_cast<BtContext>(_downloadContext);
-  if(!ctx.isNull()) {
+  if(_downloadContext->hasAttribute(bittorrent::BITTORRENT)) {
     TransferStat stat = calculateStat();
-    double shareRatio = ((stat.getAllTimeUploadLength()*10)/getCompletedLength())/10.0;
-    _logger->notice(MSG_SHARE_RATIO_REPORT,
-		    shareRatio,
-		    Util::abbrevSize(stat.getAllTimeUploadLength()).c_str(),
-		    Util::abbrevSize(getCompletedLength()).c_str());
+    double shareRatio =
+      ((stat.getAllTimeUploadLength()*10)/getCompletedLength())/10.0;
+    const BDE& attrs = _downloadContext->getAttribute(bittorrent::BITTORRENT);
+    if(attrs.containsKey(bittorrent::METADATA)) {
+      _logger->notice(MSG_SHARE_RATIO_REPORT,
+		      shareRatio,
+		      util::abbrevSize(stat.getAllTimeUploadLength()).c_str(),
+		      util::abbrevSize(getCompletedLength()).c_str());
+    }
   }
 #endif // ENABLE_BITTORRENT
 }
 
-const std::deque<std::string>& RequestGroup::getAcceptFeatures() const
-{
-  return _acceptFeatures;
-}
-
-void RequestGroup::addAcceptFeatureHeader(const std::string& feature)
-{
-  if(std::find(_acceptFeatures.begin(), _acceptFeatures.end(), feature) == _acceptFeatures.end()) {
-    _acceptFeatures.push_back(feature);
-  }
-}
-
-void RequestGroup::removeAcceptFeatureHeader(const std::string& feature)
-{
-  std::deque<std::string>::iterator i = std::find(_acceptFeatures.begin(), _acceptFeatures.end(), feature);
-  if(i != _acceptFeatures.end()) {
-    _acceptFeatures.erase(i);
-  }
-}
-
-const std::deque<std::string>& RequestGroup::getAcceptTypes() const
-{
-  return _acceptTypes;
-}
-
 void RequestGroup::addAcceptType(const std::string& type)
 {
   if(std::find(_acceptTypes.begin(), _acceptTypes.end(), type) == _acceptTypes.end()) {
@@ -1069,24 +1026,59 @@ void RequestGroup::increaseAndValidateFileNotFoundCount()
   const unsigned int maxCount = _option->getAsInt(PREF_MAX_FILE_NOT_FOUND);
   if(maxCount > 0 && _fileNotFoundCount >= maxCount &&
      _segmentMan->calculateSessionDownloadLength() == 0) {
-    throw DownloadFailureException
-      (StringFormat("Reached max-file-not-found count=%u", maxCount).str());
+    throw DOWNLOAD_FAILURE_EXCEPTION2
+      (StringFormat("Reached max-file-not-found count=%u", maxCount).str(),
+       downloadresultcode::MAX_FILE_NOT_FOUND);
   }
 }
 
-unsigned int RequestGroup::getNumConcurrentCommand() const
-{
-  return _numConcurrentCommand;
-}
-
 void RequestGroup::markInMemoryDownload()
 {
   _inMemoryDownload = true;
 }
 
-bool RequestGroup::inMemoryDownload() const
+void RequestGroup::setTimeout(time_t timeout)
 {
-  return _inMemoryDownload;
+  _timeout = timeout;
+}
+
+bool RequestGroup::doesDownloadSpeedExceed()
+{
+  return _maxDownloadSpeedLimit > 0 &&
+    _maxDownloadSpeedLimit < calculateStat().getDownloadSpeed();
+}
+
+bool RequestGroup::doesUploadSpeedExceed()
+{
+  return _maxUploadSpeedLimit > 0 &&
+    _maxUploadSpeedLimit < calculateStat().getUploadSpeed();
+}
+
+void RequestGroup::setLastUriResult
+(const std::string uri, downloadresultcode::RESULT result)
+{
+  _lastUriResult.reset(new URIResult(uri, result));
+}
+
+void RequestGroup::saveControlFile() const
+{
+  if(_saveControlFile) {
+    _progressInfoFile->save();
+  }
+}
+
+void RequestGroup::removeControlFile() const
+{
+  _progressInfoFile->removeFile();
+}
+
+void RequestGroup::setDownloadContext
+(const SharedHandle<DownloadContext>& downloadContext)
+{
+  _downloadContext = downloadContext;
+  if(!_downloadContext.isNull()) {
+    _downloadContext->setOwnerRequestGroup(this);
+  }
 }
 
 } // namespace aria2
