@@ -39,13 +39,13 @@
 
 #include <string>
 #include <deque>
+#include <vector>
 
 #include "SharedHandle.h"
 #include "TransferStat.h"
 #include "TimeA2.h"
 #include "Request.h"
-#include "DownloadResult.h"
-#include "URIResult.h"
+#include "DownloadResultCode.h"
 
 namespace aria2 {
 
@@ -66,8 +66,8 @@ class Logger;
 class RequestGroup;
 class CheckIntegrityEntry;
 class DownloadResult;
-class ServerHost;
 class URISelector;
+class URIResult;
 #ifdef ENABLE_BITTORRENT
 class BtRuntime;
 class PeerStorage;
@@ -87,10 +87,7 @@ private:
 
   SharedHandle<Option> _option;
 
-  std::deque<std::string> _uris;
-  std::deque<std::string> _spentUris;
-
-  unsigned int _numConcurrentCommand;
+  size_t _numConcurrentCommand;
 
   /**
    * This is the number of connections used in streaming protocol(http/ftp)
@@ -106,14 +103,14 @@ private:
 
   SharedHandle<PieceStorage> _pieceStorage;
 
+  bool _saveControlFile;
+
   SharedHandle<BtProgressInfoFile> _progressInfoFile;
 
   SharedHandle<DiskWriterFactory> _diskWriterFactory;
 
   SharedHandle<Dependency> _dependency;
 
-  std::deque<SharedHandle<ServerHost> > _serverHosts;
-
   bool _fileAllocationEnabled;
 
   bool _preLocalFileCheckEnabled;
@@ -124,18 +121,10 @@ private:
 
   HaltReason _haltReason;
 
-  // URIResult is stored in the ascending order of the time when its result is
-  // available.
-  std::deque<URIResult> _uriResults;
-
-  bool _singleHostMultiConnectionEnabled;
-
   std::deque<SharedHandle<PreDownloadHandler> > _preDownloadHandlers;
 
   std::deque<SharedHandle<PostDownloadHandler> > _postDownloadHandlers;
 
-  std::deque<std::string> _acceptFeatures;
-
   std::deque<std::string> _acceptTypes;
 
   SharedHandle<URISelector> _uriSelector;
@@ -147,9 +136,6 @@ private:
   // Timeout used for HTTP/FTP downloads.
   time_t _timeout;
 
-  // How many times HTTP/FTP download should retry.
-  unsigned int _maxTries;
-
 #ifdef ENABLE_BITTORRENT
   WeakHandle<BtRuntime> _btRuntime;
 
@@ -164,29 +150,40 @@ private:
 
   unsigned int _maxUploadSpeedLimit;
 
+  SharedHandle<URIResult> _lastUriResult;
+
+  // If this download generates another downloads when completed(for
+  // example, downloads generated by PostDownloadHandler), this field
+  // has the GID of generated RequestGroups. empty list means there is
+  // no such RequestGroup.
+  std::vector<int32_t> _followedByGIDs;
+
+  // If this download is a part of another download(for example,
+  // downloading torrent file described in Metalink file), this field
+  // has the GID of parent RequestGroup. 0 means this is a parent
+  // RequestGroup.
+  int32_t _belongsToGID;
+
   Logger* _logger;
 
   void validateFilename(const std::string& expectedFilename,
 			const std::string& actualFilename) const;
 
-  void validateTotalLength(uint64_t expectedTotalLength,
-			   uint64_t actualTotalLength) const;
-
   void initializePreDownloadHandler();
 
   void initializePostDownloadHandler();
 
   bool tryAutoFileRenaming();
 
-  // Returns the result code of this RequestGroup.
-  // If the download finished, then returns DownloadResult::FINISHED.
-  // If the download didn't finish and error result is available in _uriResults,
-  // then last result code is returned.
-  // Otherwise returns DownloadResult::UNKNOWN_ERROR.
-  DownloadResult::RESULT downloadResult() const;
+  // Returns the result code of this RequestGroup.  If the download
+  // finished, then returns downloadresultcode::FINISHED.  If the
+  // download didn't finish and error result is available in
+  // _uriResults, then last result code is returned.  Otherwise
+  // returns downloadresultcode::UNKNOWN_ERROR.
+  downloadresultcode::RESULT downloadResult() const;
 public:
-  RequestGroup(const SharedHandle<Option>& option,
-	       const std::deque<std::string>& uris);
+  // The copy of option is stored in RequestGroup object.
+  RequestGroup(const SharedHandle<Option>& option);
 
   ~RequestGroup();
   /**
@@ -202,51 +199,28 @@ public:
 
   // Returns first bootstrap commands to initiate a download.
   // If this is HTTP/FTP download and file size is unknown, only 1 command
-  // (usually, HttpInitiateConnection or FtpInitiateConnection) will be created
-  // with its Request object having Requet::METHOD_HEAD in its method.
-  // This behavior can be changed by providing 3rd argument.
-  // The method has effect only for using HTTP request including FTP via HTTP
-  // proxy.
+  // (usually, HttpInitiateConnection or FtpInitiateConnection) will be created.
   void createInitialCommand(std::deque<Command*>& commands,
-			    DownloadEngine* e,
-			    const std::string& method = Request::METHOD_HEAD);
+			    DownloadEngine* e);
 
   void createNextCommandWithAdj(std::deque<Command*>& commands,
 				DownloadEngine* e, int numAdj);
 
   void createNextCommand(std::deque<Command*>& commands,
-			 DownloadEngine* e, unsigned int numCommand,
-			 const std::string& method = Request::METHOD_GET);
+			 DownloadEngine* e, unsigned int numCommand);
   
-  void addURI(const std::string& uri)
-  {
-    _uris.push_back(uri);
-  }
-
   bool downloadFinished() const;
 
   bool allDownloadFinished() const;
 
   void closeFile();
 
-  std::string getFilePath() const;
+  std::string getFirstFilePath() const;
 
   uint64_t getTotalLength() const;
 
   uint64_t getCompletedLength() const;
 
-  const std::deque<std::string>& getRemainingUris() const
-  {
-    return _uris;
-  }
-
-  const std::deque<std::string>& getSpentUris() const
-  {
-    return _spentUris;
-  }
-
-  void getURIs(std::deque<std::string>& uris) const;
-
   /**
    * Compares expected filename with specified actualFilename.
    * The expected filename refers to FileEntry::getBasename() of the first
@@ -254,6 +228,9 @@ public:
    */
   void validateFilename(const std::string& actualFilename) const;
 
+  void validateTotalLength(uint64_t expectedTotalLength,
+			   uint64_t actualTotalLength) const;
+
   void validateTotalLength(uint64_t actualTotalLength) const;
 
   void setSegmentManFactory(const SharedHandle<SegmentManFactory>& segmentManFactory);
@@ -280,6 +257,8 @@ public:
     return _downloadContext;
   }
 
+  // This function also calls
+  // downloadContext->setOwnerRequestGroup(this).
   void setDownloadContext(const SharedHandle<DownloadContext>& downloadContext);
 
   const SharedHandle<PieceStorage>& getPieceStorage() const
@@ -289,17 +268,15 @@ public:
 
   void setPieceStorage(const SharedHandle<PieceStorage>& pieceStorage);
 
-  const SharedHandle<BtProgressInfoFile>& getProgressInfoFile() const
-  {
-    return _progressInfoFile;
-  }
-
   void setProgressInfoFile(const SharedHandle<BtProgressInfoFile>& progressInfoFile);
 
   void increaseStreamConnection();
 
   void decreaseStreamConnection();
 
+  // Returns the number of connections used in HTTP(S)/FTP.
+  unsigned int getNumStreamConnection() { return _numStreamConnection; }
+
   unsigned int getNumConnection() const;
 
   void increaseNumCommand();
@@ -360,17 +337,6 @@ public:
     return _forceHaltRequested;
   }
 
-  void addURIResult(std::string uri, DownloadResult::RESULT result);
-
-  const std::deque<URIResult>& getURIResults() const
-  {
-    return _uriResults;
-  }
-
-  // Extracts URIResult whose _result is r and stores them into res.
-  // The extracted URIResults are removed from _uriResults.
-  void extractURIResult(std::deque<URIResult>& res, DownloadResult::RESULT r);
-
   void dependsOn(const SharedHandle<Dependency>& dep);
 
   bool isDependencyResolved();
@@ -381,13 +347,13 @@ public:
 
   void addPostDownloadHandler(const SharedHandle<PostDownloadHandler>& handler);
 
-  void clearPostDowloadHandler();
+  void clearPostDownloadHandler();
 
   void preDownloadProcessing();
 
   void addPreDownloadHandler(const SharedHandle<PreDownloadHandler>& handler);
 
-  void clearPreDowloadHandler();
+  void clearPreDownloadHandler();
 
   void processCheckIntegrityEntry(std::deque<Command*>& commands,
 				  const SharedHandle<CheckIntegrityEntry>& entry,
@@ -410,46 +376,8 @@ public:
     return _option;
   }
 
-  bool isSingleHostMultiConnectionEnabled() const
-  {
-    return _singleHostMultiConnectionEnabled;
-  }
-
-  void setSingleHostMultiConnectionEnabled(bool f)
-  {
-    _singleHostMultiConnectionEnabled = f;
-  }
-
-  /**
-   * Registers given ServerHost.
-   */
-  void registerServerHost(const SharedHandle<ServerHost>& serverHost);
-
-  /**
-   * Returns ServerHost whose cuid is given cuid. If it is not found, returns
-   * 0.
-   */
-  SharedHandle<ServerHost> searchServerHost(int32_t cuid) const;
-
-  SharedHandle<ServerHost> searchServerHost(const std::string& hostname) const;
-
-  void removeServerHost(int32_t cuid);
-  
-  void removeURIWhoseHostnameIs(const std::string& hostname);
-
-  void removeIdenticalURI(const std::string& uri);
-
   void reportDownloadFinished();
 
-  const std::deque<std::string>& getAcceptFeatures() const
-  {
-    return _acceptFeatures;
-  }
-
-  void addAcceptFeatureHeader(const std::string& feature);
-
-  void removeAcceptFeatureHeader(const std::string& feature);
-
   const std::deque<std::string>& getAcceptTypes() const
   {
     return _acceptTypes;
@@ -463,6 +391,11 @@ public:
 
   void setURISelector(const SharedHandle<URISelector>& uriSelector);
 
+  const SharedHandle<URISelector>& getURISelector() const
+  {
+    return _uriSelector;
+  }
+
   void applyLastModifiedTimeToLocalFiles();
 
   void updateLastModifiedTime(const Time& time);
@@ -478,8 +411,6 @@ public:
     return _inMemoryDownload;
   }
 
-  void tuneDownloadCommand(DownloadCommand* command);
-
   void setTimeout(time_t timeout);
 
   time_t getTimeout() const
@@ -487,13 +418,6 @@ public:
     return _timeout;
   }
 
-  void setMaxTries(unsigned int maxTries);
-
-  unsigned int getMaxTries() const
-  {
-    return _maxTries;
-  }
-
   // Returns true if current download speed exceeds
   // _maxDownloadSpeedLimit.  Always returns false if
   // _maxDownloadSpeedLimit == 0.  Otherwise returns false.
@@ -524,6 +448,40 @@ public:
     _maxUploadSpeedLimit = speed;
   }
 
+  void setLastUriResult(std::string uri, downloadresultcode::RESULT result);
+
+  void saveControlFile() const;
+
+  void removeControlFile() const;
+
+  void enableSaveControlFile() { _saveControlFile = true; }
+
+  void disableSaveControlFile() { _saveControlFile = false; }
+
+  template<typename InputIterator>
+  void followedBy(InputIterator groupFirst, InputIterator groupLast)
+  {
+    _followedByGIDs.clear();
+    for(; groupFirst != groupLast; ++groupFirst) {
+      _followedByGIDs.push_back((*groupFirst)->getGID());
+    }
+  }
+
+  const std::vector<int32_t>& followedBy() const
+  {
+    return _followedByGIDs;
+  }
+
+  void belongsTo(int32_t gid)
+  {
+    _belongsToGID = gid;
+  }
+
+  int32_t belongsTo() const
+  {
+    return _belongsToGID;
+  }
+
   static void resetGIDCounter() { _gidCounter = 0; }
 };
 
