@@ -73,6 +73,9 @@ private:
 
   unsigned int _maxOverallUploadSpeedLimit;
 
+  // truf if XML-RPC is enabled.
+  bool _xmlRpc;
+
   std::string
   formatDownloadResult(const std::string& status,
 		       const SharedHandle<DownloadResult>& downloadResult) const;
@@ -94,8 +97,6 @@ public:
 
   void forceHalt();
 
-  void getInitialCommands(std::deque<Command*>& commands, DownloadEngine* e);
-
   void removeStoppedGroup(DownloadEngine* e);
 
   void fillRequestGroupFromReserver(DownloadEngine* e);
@@ -129,6 +130,23 @@ public:
 
   SharedHandle<RequestGroup> findReservedGroup(int32_t gid) const;
 
+  enum HOW {
+    POS_SET,
+    POS_CUR,
+    POS_END
+  };
+
+  // Changes the position of download denoted by gid.  If how is
+  // POS_SET, it moves the download to a position relative to the
+  // beginning of the queue.  If how is POS_CUR, it moves the download
+  // to a position relative to the current position. If how is
+  // POS_END, it moves the download to a position relative to the end
+  // of the queue. If the destination position is less than 0 or
+  // beyond the end of the queue, it moves the download to the
+  // beginning or the end of the queue respectively.  Returns the
+  // destination position.
+  size_t changeReservedGroupPosition(int32_t gid, int pos, HOW how);
+
   bool removeReservedGroup(int32_t gid);
 
   void showDownloadResults(std::ostream& o) const;
@@ -143,21 +161,21 @@ public:
     size_t _error;
     size_t _inProgress;
     size_t _waiting;
-    DownloadResult::RESULT _lastErrorResult;
+    downloadresultcode::RESULT _lastErrorResult;
   public:
     DownloadStat(size_t completed,
 		 size_t error,
 		 size_t inProgress,
 		 size_t waiting,
-		 DownloadResult::RESULT lastErrorResult =
-		 DownloadResult::FINISHED):
+		 downloadresultcode::RESULT lastErrorResult =
+		 downloadresultcode::FINISHED):
       _completed(completed),
       _error(error),
       _inProgress(inProgress),
       _waiting(waiting),
       _lastErrorResult(lastErrorResult) {}
 
-    DownloadResult::RESULT getLastErrorResult() const
+    downloadresultcode::RESULT getLastErrorResult() const
     {
       return _lastErrorResult;
     }
