@@ -38,19 +38,16 @@
 
 #include "DownloadEngine.h"
 #include "Option.h"
-#include "Request.h"
 #include "prefs.h"
 #include "RequestGroup.h"
-#include "InitiateConnectionCommandFactory.h"
 #include "DownloadContext.h"
+#include "Command.h"
 
 namespace aria2 {
 
-StreamFileAllocationEntry::StreamFileAllocationEntry(const RequestHandle& currentRequest,
-						     RequestGroup* requestGroup,
+StreamFileAllocationEntry::StreamFileAllocationEntry(RequestGroup* requestGroup,
 						     Command* nextCommand):
-  FileAllocationEntry(requestGroup, nextCommand),
-  _currentRequest(currentRequest)
+  FileAllocationEntry(requestGroup, nextCommand)
 {}
 
 StreamFileAllocationEntry::~StreamFileAllocationEntry() {}
@@ -62,21 +59,12 @@ void StreamFileAllocationEntry::prepareForNextAction(std::deque<Command*>& comma
   if(_nextCommand) {
     // give _nextCommand a chance to execute in the next execution loop.
     _nextCommand->setStatus(Command::STATUS_ONESHOT_REALTIME);
+    e->setNoWait(true);
     commands.push_back(popNextCommand());
     // try remaining uris
     _requestGroup->createNextCommandWithAdj(commands, e, -1);
   } else {
-    if(_currentRequest.isNull()) {
-      _requestGroup->createNextCommandWithAdj(commands, e, 0);
-    } else {
-      Command* command =
-	InitiateConnectionCommandFactory::createInitiateConnectionCommand
-	(e->newCUID(), _currentRequest, _requestGroup, e);
-
-      commands.push_back(command);
-
-      _requestGroup->createNextCommandWithAdj(commands, e, -1);
-    }
+    _requestGroup->createNextCommandWithAdj(commands, e, 0);
   }
 }
 
