@@ -33,6 +33,9 @@
  */
 /* copyright --> */
 #include "UnknownLengthPieceStorage.h"
+
+#include <cstdlib>
+
 #include "DefaultDiskWriter.h"
 #include "DirectDiskAdaptor.h"
 #include "prefs.h"
@@ -40,12 +43,12 @@
 #include "DownloadContext.h"
 #include "Piece.h"
 #include "FileEntry.h"
-#include <stdlib.h>
 
 namespace aria2 {
 
-UnknownLengthPieceStorage::UnknownLengthPieceStorage(const DownloadContextHandle& downloadContext,
-						     const Option* option):
+UnknownLengthPieceStorage::UnknownLengthPieceStorage
+(const SharedHandle<DownloadContext>& downloadContext,
+ const Option* option):
   _downloadContext(downloadContext),
   _option(option),
   _diskWriterFactory(new DefaultDiskWriterFactory()),
@@ -56,19 +59,20 @@ UnknownLengthPieceStorage::~UnknownLengthPieceStorage() {}
 
 void UnknownLengthPieceStorage::initStorage()
 {
-  DiskWriterHandle writer = _diskWriterFactory->newDiskWriter();
   DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
-  directDiskAdaptor->setDiskWriter(writer);
   directDiskAdaptor->setTotalLength(_downloadContext->getTotalLength());
+  directDiskAdaptor->setFileEntries(_downloadContext->getFileEntries().begin(),
+				    _downloadContext->getFileEntries().end());
+
+  DiskWriterHandle writer =
+    _diskWriterFactory->newDiskWriter(directDiskAdaptor->getFilePath());
+  directDiskAdaptor->setDiskWriter(writer);
+
   _diskAdaptor = directDiskAdaptor;
-  std::string storeDir = _downloadContext->getDir();
-//   if(storeDir == "") {
-//     storeDir = ".";
-//   }
-  _diskAdaptor->setStoreDir(storeDir);
-  _diskAdaptor->setFileEntries(_downloadContext->getFileEntries());
 }
 
+#ifdef ENABLE_BITTORRENT
+
 bool UnknownLengthPieceStorage::hasMissingPiece(const SharedHandle<Peer>& peer)
 {
   abort();
@@ -96,7 +100,15 @@ SharedHandle<Piece> UnknownLengthPieceStorage::getMissingFastPiece
   abort();
 }
 
-PieceHandle UnknownLengthPieceStorage::getMissingPiece()
+#endif // ENABLE_BITTORRENT
+
+bool UnknownLengthPieceStorage::hasMissingUnusedPiece()
+{
+  abort();
+}
+
+SharedHandle<Piece> UnknownLengthPieceStorage::getSparseMissingUnusedPiece
+(const unsigned char* ignoreBitfield, size_t length)
 {
   if(_downloadFinished) {
     return SharedHandle<Piece>();
@@ -112,7 +124,7 @@ PieceHandle UnknownLengthPieceStorage::getMissingPiece()
 PieceHandle UnknownLengthPieceStorage::getMissingPiece(size_t index)
 {
   if(index == 0) {
-    return getMissingPiece();
+    return getSparseMissingUnusedPiece(0, 0);
   } else {
     return SharedHandle<Piece>();
   }
