@@ -46,39 +46,80 @@
 #include "DownloadEngine.h"
 #include "RequestGroup.h"
 #include "download_helper.h"
-#include "Util.h"
+#include "util.h"
 #include "RequestGroupMan.h"
 #include "StringFormat.h"
 #include "XmlRpcRequest.h"
 #include "PieceStorage.h"
-#include "PeerStorage.h"
-#include "BtContext.h"
-#include "BtRegistry.h"
-#include "Peer.h"
+#include "DownloadContext.h"
 #include "DiskAdaptor.h"
 #include "FileEntry.h"
 #include "BtProgressInfoFile.h"
-#include "BtRuntime.h"
-#include "BtAnnounce.h"
 #include "prefs.h"
 #include "message.h"
+#include "FeatureConfig.h"
+#include "array_fun.h"
+#include "XmlRpcMethodFactory.h"
+#include "XmlRpcResponse.h"
+#ifdef ENABLE_BITTORRENT
+# include "bittorrent_helper.h"
+# include "BtRegistry.h"
+# include "PeerStorage.h"
+# include "Peer.h"
+# include "BtRuntime.h"
+# include "BtAnnounce.h"
+#endif // ENABLE_BITTORRENT
 
 namespace aria2 {
 
 namespace xmlrpc {
 
-static const BDE BDE_TRUE = BDE("true");
-static const BDE BDE_FALSE = BDE("false");
-static const BDE BDE_OK = BDE("OK");
-static const BDE BDE_ACTIVE = BDE("active");
-static const BDE BDE_WAITING = BDE("waiting");
-static const BDE BDE_REMOVED = BDE("removed");
-static const BDE BDE_ERROR = BDE("error");
-static const BDE BDE_COMPLETE = BDE("complete");
+namespace {
+const BDE BDE_TRUE = BDE("true");
+const BDE BDE_FALSE = BDE("false");
+const BDE BDE_OK = BDE("OK");
+const BDE BDE_ACTIVE = BDE("active");
+const BDE BDE_WAITING = BDE("waiting");
+const BDE BDE_REMOVED = BDE("removed");
+const BDE BDE_ERROR = BDE("error");
+const BDE BDE_COMPLETE = BDE("complete");
+
+const std::string KEY_GID = "gid";
+const std::string KEY_ERROR_CODE = "errorCode";
+const std::string KEY_STATUS = "status";
+const std::string KEY_TOTAL_LENGTH = "totalLength";
+const std::string KEY_COMPLETED_LENGTH = "completedLength";
+const std::string KEY_DOWNLOAD_SPEED = "downloadSpeed";
+const std::string KEY_UPLOAD_SPEED = "uploadSpeed";
+const std::string KEY_UPLOAD_LENGTH = "uploadLength";
+const std::string KEY_CONNECTIONS = "connections";
+const std::string KEY_BITFIELD = "bitfield";
+const std::string KEY_PIECE_LENGTH = "pieceLength";
+const std::string KEY_NUM_PIECES = "numPieces";
+const std::string KEY_FOLLOWED_BY = "followedBy";
+const std::string KEY_BELONGS_TO = "belongsTo";
+const std::string KEY_INFO_HASH = "infoHash";
+const std::string KEY_NUM_SEEDERS = "numSeeders";
+const std::string KEY_PEER_ID = "peerId";
+const std::string KEY_IP = "ip";
+const std::string KEY_PORT = "port";
+const std::string KEY_AM_CHOKING = "amChoking";
+const std::string KEY_PEER_CHOKING = "peerChoking";
+const std::string KEY_SEEDER = "seeder";
+const std::string KEY_INDEX = "index";
+const std::string KEY_PATH = "path";
+const std::string KEY_SELECTED = "selected";
+const std::string KEY_LENGTH = "length";
+const std::string KEY_URI = "uri";
+const std::string KEY_VERSION = "version";
+const std::string KEY_ENABLED_FEATURES = "enabledFeatures";
+const std::string KEY_METHOD_NAME = "methodName";
+const std::string KEY_PARAMS = "params";
+}
 
 static BDE createGIDResponse(int32_t gid)
 {
-  return BDE(Util::itos(gid));
+  return BDE(util::itos(gid));
 }
 
 static BDE addRequestGroup(const SharedHandle<RequestGroup>& group,
@@ -102,8 +143,12 @@ static void getPosParam(const BDE& params, size_t posParamIndex,
 			bool& posGiven, size_t& pos)
 {
   if(params.size() > posParamIndex && params[posParamIndex].isInteger()) {
-    pos = params[posParamIndex].i();
-    posGiven = true;
+    if(params[posParamIndex].i() >= 0) {
+      pos = params[posParamIndex].i();
+      posGiven = true;
+    } else {
+      throw DL_ABORT_EX("Position must be greater than or equal to 0.");
+    }
   } else {
     posGiven = false;
   }
@@ -135,7 +180,7 @@ BDE AddUriXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
   std::deque<SharedHandle<RequestGroup> > result;
   createRequestGroupForUri(result, requestOption, uris,
 			   /* ignoreForceSeq = */ true,
-			   /* ignoreNonURI = */ true);
+			   /* ignoreLocalPath = */ true);
 
   if(!result.empty()) {
     return addRequestGroup(result.front(), e, posGiven, pos);
@@ -213,7 +258,7 @@ BDE AddMetalinkXmlRpcMethod::process
     BDE gids = BDE::list();
     for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
 	  result.begin(); i != result.end(); ++i) {
-      gids << BDE(Util::itos((*i)->getGID()));
+      gids << BDE(util::itos((*i)->getGID()));
     }
     return gids;
   } else {
@@ -231,7 +276,7 @@ BDE RemoveXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = Util::parseInt(params[0].s());
+  int32_t gid = util::parseInt(params[0].s());
 
   SharedHandle<RequestGroup> group = e->_requestGroupMan->findRequestGroup(gid);
 
@@ -254,46 +299,59 @@ BDE RemoveXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
   return createGIDResponse(gid);
 }
 
-static void gatherProgressCommon
+void gatherProgressCommon
 (BDE& entryDict, const SharedHandle<RequestGroup>& group)
 {
-  entryDict["gid"] = Util::itos(group->getGID());
+  entryDict[KEY_GID] = util::itos(group->getGID());
   // This is "filtered" total length if --select-file is used.
-  entryDict["totalLength"] = Util::uitos(group->getTotalLength());
+  entryDict[KEY_TOTAL_LENGTH] = util::uitos(group->getTotalLength());
   // This is "filtered" total length if --select-file is used.
-  entryDict["completedLength"] = Util::uitos(group->getCompletedLength());
+  entryDict[KEY_COMPLETED_LENGTH] = util::uitos(group->getCompletedLength());
   TransferStat stat = group->calculateStat();
-  entryDict["downloadSpeed"] = Util::uitos(stat.getDownloadSpeed());
-  entryDict["uploadSpeed"] = Util::uitos(stat.getUploadSpeed());
-  entryDict["uploadLength"] = Util::uitos(stat.getAllTimeUploadLength());
-  entryDict["connections"] = Util::uitos(group->getNumConnection());
+  entryDict[KEY_DOWNLOAD_SPEED] = util::uitos(stat.getDownloadSpeed());
+  entryDict[KEY_UPLOAD_SPEED] = util::uitos(stat.getUploadSpeed());
+  entryDict[KEY_UPLOAD_LENGTH] = util::uitos(stat.getAllTimeUploadLength());
+  entryDict[KEY_CONNECTIONS] = util::uitos(group->getNumConnection());
   SharedHandle<PieceStorage> ps = group->getPieceStorage();
   if(!ps.isNull()) {
     if(ps->getBitfieldLength() > 0) {
-      entryDict["bitfield"] = Util::toHex(ps->getBitfield(),
-					  ps->getBitfieldLength());
+      entryDict[KEY_BITFIELD] = util::toHex(ps->getBitfield(),
+					    ps->getBitfieldLength());
     }
   }
-  entryDict["pieceLength"] = 
-    Util::uitos(group->getDownloadContext()->getPieceLength());
-  entryDict["numPieces"] =
-    Util::uitos(group->getDownloadContext()->getNumPieces());
+  entryDict[KEY_PIECE_LENGTH] = 
+    util::uitos(group->getDownloadContext()->getPieceLength());
+  entryDict[KEY_NUM_PIECES] =
+    util::uitos(group->getDownloadContext()->getNumPieces());
+  if(!group->followedBy().empty()) {
+    BDE list = BDE::list();
+    // The element is GID.
+    for(std::vector<int32_t>::const_iterator i = group->followedBy().begin();
+	i != group->followedBy().end(); ++i) {
+      list << util::itos(*i);
+    }
+    entryDict[KEY_FOLLOWED_BY] = list;
+  }
+  if(group->belongsTo()) {
+    entryDict[KEY_BELONGS_TO] = util::itos(group->belongsTo());
+  }
 }
 
 #ifdef ENABLE_BITTORRENT
 static void gatherProgressBitTorrent
-(BDE& entryDict, const SharedHandle<BtContext>& btctx,
- const SharedHandle<BtRegistry>& btreg)
+(BDE& entryDict, const BDE& torrentAttrs, const BtObject& btObject)
 {
-  entryDict["infoHash"] = btctx->getInfoHashAsString();
+  const std::string& infoHash = torrentAttrs[bittorrent::INFO_HASH].s();
+  entryDict[KEY_INFO_HASH] = util::toHex(infoHash);
 
-  SharedHandle<PeerStorage> peerStorage =
-    btreg->getPeerStorage(btctx->getInfoHashAsString());
-  assert(!peerStorage.isNull());
+  if(!btObject.isNull()) {
+    SharedHandle<PeerStorage> peerStorage = btObject._peerStorage;
+    assert(!peerStorage.isNull());
 
-  std::deque<SharedHandle<Peer> > peers;
-  peerStorage->getActivePeers(peers);
-  entryDict["numSeeders"] = countSeeder(peers.begin(), peers.end());
+    std::deque<SharedHandle<Peer> > peers;
+    peerStorage->getActivePeers(peers);
+    entryDict[KEY_NUM_SEEDERS] = countSeeder(peers.begin(), peers.end());
+  }
 }
 
 static void gatherPeer(BDE& peers, const SharedHandle<PeerStorage>& ps)
@@ -303,18 +361,18 @@ static void gatherPeer(BDE& peers, const SharedHandle<PeerStorage>& ps)
   for(std::deque<SharedHandle<Peer> >::const_iterator i =
 	activePeers.begin(); i != activePeers.end(); ++i) {
     BDE peerEntry = BDE::dict();
-    peerEntry["peerId"] = Util::torrentUrlencode((*i)->getPeerId(),
-						 PEER_ID_LENGTH);
-    peerEntry["ip"] = (*i)->ipaddr;
-    peerEntry["port"] = Util::uitos((*i)->port);
-    peerEntry["bitfield"] = Util::toHex((*i)->getBitfield(),
+    peerEntry[KEY_PEER_ID] = util::torrentUrlencode((*i)->getPeerId(),
+						PEER_ID_LENGTH);
+    peerEntry[KEY_IP] = (*i)->ipaddr;
+    peerEntry[KEY_PORT] = util::uitos((*i)->port);
+    peerEntry[KEY_BITFIELD] = util::toHex((*i)->getBitfield(),
 					(*i)->getBitfieldLength());
-    peerEntry["amChoking"] = (*i)->amChoking()?BDE_TRUE:BDE_FALSE;
-    peerEntry["peerChoking"] = (*i)->peerChoking()?BDE_TRUE:BDE_FALSE;
+    peerEntry[KEY_AM_CHOKING] = (*i)->amChoking()?BDE_TRUE:BDE_FALSE;
+    peerEntry[KEY_PEER_CHOKING] = (*i)->peerChoking()?BDE_TRUE:BDE_FALSE;
     TransferStat stat = ps->getTransferStatFor(*i);
-    peerEntry["downloadSpeed"] = Util::uitos(stat.getDownloadSpeed());
-    peerEntry["uploadSpeed"] = Util::uitos(stat.getUploadSpeed());
-    peerEntry["seeder"] = (*i)->isSeeder()?BDE_TRUE:BDE_FALSE;
+    peerEntry[KEY_DOWNLOAD_SPEED] = util::uitos(stat.getDownloadSpeed());
+    peerEntry[KEY_UPLOAD_SPEED] = util::uitos(stat.getUploadSpeed());
+    peerEntry[KEY_SEEDER] = (*i)->isSeeder()?BDE_TRUE:BDE_FALSE;
     peers << peerEntry;
   }
 }
@@ -325,25 +383,38 @@ static void gatherProgress
 {
   gatherProgressCommon(entryDict, group);
 #ifdef ENABLE_BITTORRENT
-  SharedHandle<BtContext> btctx =
-    dynamic_pointer_cast<BtContext>(group->getDownloadContext());
-  if(!btctx.isNull()) {
-    SharedHandle<BtRegistry> btreg = e->getBtRegistry();
-    gatherProgressBitTorrent(entryDict, btctx, btreg);
+  if(group->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT)) {
+    const BDE& torrentAttrs =
+      group->getDownloadContext()->getAttribute(bittorrent::BITTORRENT);
+    BtObject btObject = e->getBtRegistry()->get(group->getGID());
+    gatherProgressBitTorrent(entryDict, torrentAttrs, btObject);
   }
 #endif // ENABLE_BITTORRENT
 }
 
-static void gatherStoppedDownload
+void gatherStoppedDownload
 (BDE& entryDict, const SharedHandle<DownloadResult>& ds)
 {
-  entryDict["gid"] = Util::itos(ds->gid);
-  if(ds->result == DownloadResult::IN_PROGRESS) {
-    entryDict["status"] = BDE_REMOVED;
-  } else if(ds->result == DownloadResult::FINISHED) {
-    entryDict["status"] = BDE_COMPLETE;
+  entryDict[KEY_GID] = util::itos(ds->gid);
+  entryDict[KEY_ERROR_CODE] = util::itos(static_cast<int>(ds->result));
+  if(ds->result == downloadresultcode::IN_PROGRESS) {
+    entryDict[KEY_STATUS] = BDE_REMOVED;
+  } else if(ds->result == downloadresultcode::FINISHED) {
+    entryDict[KEY_STATUS] = BDE_COMPLETE;
   } else {
-    entryDict["status"] = BDE_ERROR;
+    entryDict[KEY_STATUS] = BDE_ERROR;
+  }
+  if(!ds->followedBy.empty()) {
+    BDE list = BDE::list();
+    // The element is GID.
+    for(std::vector<int32_t>::const_iterator i = ds->followedBy.begin();
+	i != ds->followedBy.end(); ++i) {
+      list << util::itos(*i);
+    }
+    entryDict[KEY_FOLLOWED_BY] = list;
+  }
+  if(ds->belongsTo) {
+    entryDict[KEY_BELONGS_TO] = util::itos(ds->belongsTo);
   }
 }
 
@@ -364,10 +435,10 @@ static void createFileEntry(BDE& files, InputIterator first, InputIterator last)
   size_t index = 1;
   for(; first != last; ++first, ++index) {
     BDE entry = BDE::dict();
-    entry["index"] = Util::uitos(index);
-    entry["path"] = (*first)->getPath();
-    entry["selected"] = (*first)->isRequested()?BDE_TRUE:BDE_FALSE;
-    entry["length"] = Util::uitos((*first)->getLength());
+    entry[KEY_INDEX] = util::uitos(index);
+    entry[KEY_PATH] = (*first)->getPath();
+    entry[KEY_SELECTED] = (*first)->isRequested()?BDE_TRUE:BDE_FALSE;
+    entry[KEY_LENGTH] = util::uitos((*first)->getLength());
     files << entry;
   }
 }
@@ -382,7 +453,7 @@ BDE GetFilesXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = Util::parseInt(params[0].s());
+  int32_t gid = util::parseInt(params[0].s());
 
   BDE files = BDE::list();
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
@@ -396,9 +467,9 @@ BDE GetFilesXmlRpcMethod::process
       createFileEntry(files, dr->fileEntries.begin(), dr->fileEntries.end());
     }
   } else {
-    std::deque<SharedHandle<FileEntry> > fileEntries =
-      group->getDownloadContext()->getFileEntries();
-    createFileEntry(files, fileEntries.begin(), fileEntries.end());
+    createFileEntry(files,
+		    group->getDownloadContext()->getFileEntries().begin(),
+		    group->getDownloadContext()->getFileEntries().end());
   }
   return files;
 }
@@ -413,7 +484,7 @@ BDE GetUrisXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = Util::parseInt(params[0].s());
+  int32_t gid = util::parseInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
@@ -422,12 +493,15 @@ BDE GetUrisXmlRpcMethod::process
   }
   BDE uriList = BDE::list();
   std::deque<std::string> uris;
-  group->getURIs(uris);
-  for(std::deque<std::string>::const_iterator i = uris.begin(); i != uris.end();
-      ++i) {
-    BDE entry = BDE::dict();
-    entry["uri"] = *i;
-    uriList << entry;
+  // TODO Current implementation just returns first FileEntry's URIs.
+  if(!group->getDownloadContext()->getFileEntries().empty()) {
+    group->getDownloadContext()->getFirstFileEntry()->getUris(uris);
+    for(std::deque<std::string>::const_iterator i = uris.begin();
+	i != uris.end(); ++i) {
+      BDE entry = BDE::dict();
+      entry[KEY_URI] = *i;
+      uriList << entry;
+    }
   }
   return uriList;
 }
@@ -443,7 +517,7 @@ BDE GetPeersXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = Util::parseInt(params[0].s());
+  int32_t gid = util::parseInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
@@ -451,15 +525,10 @@ BDE GetPeersXmlRpcMethod::process
       (StringFormat("No peer data is available for GID#%d", gid).str());
   }
   BDE peers = BDE::list();
-  SharedHandle<BtContext> btctx =
-    dynamic_pointer_cast<BtContext>(group->getDownloadContext());
-  if(!btctx.isNull()) {
-    SharedHandle<BtRegistry> btreg = e->getBtRegistry();
-    SharedHandle<PeerStorage> peerStorage =
-      btreg->getPeerStorage(btctx->getInfoHashAsString());
-    assert(!peerStorage.isNull());
-    BDE entry = BDE::dict();
-    gatherPeer(peers, peerStorage);
+  BtObject btObject = e->getBtRegistry()->get(group->getGID());
+  if(!btObject.isNull()) {
+    assert(!btObject._peerStorage.isNull());
+    gatherPeer(peers, btObject._peerStorage);
   }
   return peers;
 }
@@ -475,7 +544,7 @@ BDE TellStatusXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = Util::parseInt(params[0].s());
+  int32_t gid = util::parseInt(params[0].s());
 
   SharedHandle<RequestGroup> group = e->_requestGroupMan->findRequestGroup(gid);
 
@@ -491,11 +560,11 @@ BDE TellStatusXmlRpcMethod::process
       }
       gatherStoppedDownload(entryDict, ds);
     } else {
-      entryDict["status"] = BDE_WAITING;
+      entryDict[KEY_STATUS] = BDE_WAITING;
       gatherProgress(entryDict, group, e);
     }
   } else {
-    entryDict["status"] = BDE_ACTIVE;
+    entryDict[KEY_STATUS] = BDE_ACTIVE;
     gatherProgress(entryDict, group, e);
   }
   return entryDict;
@@ -510,13 +579,82 @@ BDE TellActiveXmlRpcMethod::process
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
 	groups.begin(); i != groups.end(); ++i) {
     BDE entryDict = BDE::dict();
-    entryDict["status"] = BDE_ACTIVE;
+    entryDict[KEY_STATUS] = BDE_ACTIVE;
     gatherProgress(entryDict, *i, e);
     list << entryDict;
   }
   return list;
 }
 
+template<typename InputIterator>
+static std::pair<InputIterator, InputIterator>
+getPaginationRange
+(const XmlRpcRequest& req, InputIterator first, InputIterator last)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+
+  if(params.size() != 2 ||
+     !params[0].isInteger() || !params[1].isInteger() ||
+     params[0].i() < 0 || params[1].i() < 0) {
+    throw DL_ABORT_EX("Invalid argument. Specify offset and num in integer.");
+  }
+
+  size_t offset = params[0].i();
+  size_t num = params[1].i();
+
+  BDE list = BDE::list();
+  size_t size = std::distance(first, last);
+  if(size <= offset) {
+    return std::make_pair(last, last);
+  }
+  size_t lastDistance;
+  if(size < offset+num) {
+    lastDistance = size;
+  } else {
+    lastDistance = offset+num;
+  }
+  last = first;
+  std::advance(first, offset);
+  std::advance(last, lastDistance);
+  return std::make_pair(first, last);
+}
+
+BDE TellWaitingXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const std::deque<SharedHandle<RequestGroup> >& waitings =
+    e->_requestGroupMan->getReservedGroups();
+  std::pair<std::deque<SharedHandle<RequestGroup> >::const_iterator,
+    std::deque<SharedHandle<RequestGroup> >::const_iterator> range =
+    getPaginationRange(req, waitings.begin(), waitings.end());
+  BDE list = BDE::list();
+  for(; range.first != range.second; ++range.first) {
+    BDE entryDict = BDE::dict();
+    entryDict[KEY_STATUS] = BDE_WAITING;
+    gatherProgress(entryDict, *range.first, e);
+    list << entryDict;
+  }
+  return list;
+}
+
+BDE TellStoppedXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const std::deque<SharedHandle<DownloadResult> >& stopped =
+    e->_requestGroupMan->getDownloadResults();
+  std::pair<std::deque<SharedHandle<DownloadResult> >::const_iterator,
+    std::deque<SharedHandle<DownloadResult> >::const_iterator> range =
+    getPaginationRange(req, stopped.begin(), stopped.end());
+  BDE list = BDE::list();
+  for(; range.first != range.second; ++range.first) {
+    BDE entryDict = BDE::dict();
+    gatherStoppedDownload(entryDict, *range.first);
+    list << entryDict;
+  }
+  return list;
+}
+
 BDE PurgeDownloadResultXmlRpcMethod::process
 (const XmlRpcRequest& req, DownloadEngine* e)
 {
@@ -532,22 +670,32 @@ BDE ChangeOptionXmlRpcMethod::process
   if(params.empty() || !params[0].isString()) {
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }  
-  int32_t gid = Util::parseInt(params[0].s());
+  int32_t gid = util::parseInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
     throw DL_ABORT_EX
       (StringFormat("Cannot change option for GID#%d", gid).str());
   }
-  SharedHandle<Option> option(new Option(*group->getOption().get()));
+  SharedHandle<Option> option(new Option());
   if(params.size() > 1 && params[1].isDict()) {
     gatherChangeableOption(option, params[1]);
-  }
-  if(option->defined(PREF_MAX_DOWNLOAD_LIMIT)) {
-    group->setMaxDownloadSpeedLimit(option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
-  }
-  if(option->defined(PREF_MAX_UPLOAD_LIMIT)) {
-    group->setMaxUploadSpeedLimit(option->getAsInt(PREF_MAX_UPLOAD_LIMIT));
+    applyChangeableOption(group->getOption().get(), option.get());
+    if(option->defined(PREF_MAX_DOWNLOAD_LIMIT)) {
+      group->setMaxDownloadSpeedLimit
+	(option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
+    }
+    if(option->defined(PREF_MAX_UPLOAD_LIMIT)) {
+      group->setMaxUploadSpeedLimit(option->getAsInt(PREF_MAX_UPLOAD_LIMIT));
+    }
+#ifdef ENABLE_BITTORRENT
+    BtObject btObject = e->getBtRegistry()->get(group->getGID());
+    if(!btObject.isNull()) {
+      if(option->defined(PREF_BT_MAX_PEERS)) {
+	btObject._btRuntime->setMaxPeers(option->getAsInt(PREF_BT_MAX_PEERS));
+      }
+    }
+#endif // ENABLE_BITTORRENT
   }
   return BDE_OK;
 }
@@ -560,8 +708,10 @@ BDE ChangeGlobalOptionXmlRpcMethod::process
   if(params.empty() || !params[0].isDict()) {
     return BDE_OK;
   }
-  SharedHandle<Option> option(new Option(*e->option));
+  SharedHandle<Option> option(new Option());
   gatherChangeableGlobalOption(option, params[0]);
+  applyChangeableGlobalOption(e->option, option.get());
+
   if(option->defined(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)) {
     e->_requestGroupMan->setMaxOverallDownloadSpeedLimit
       (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT));
@@ -577,6 +727,142 @@ BDE ChangeGlobalOptionXmlRpcMethod::process
   return BDE_OK;
 }
 
+BDE GetVersionXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  BDE result = BDE::dict();
+  result[KEY_VERSION] = std::string(PACKAGE_VERSION);
+  BDE featureList = BDE::list();
+  const FeatureMap& features = FeatureConfig::getInstance()->getFeatures();
+  for(FeatureMap::const_iterator i = features.begin(); i != features.end();++i){
+    if((*i).second) {
+      featureList << (*i).first;
+    }
+  }
+  result[KEY_ENABLED_FEATURES] = featureList;
+  return result;
+}
+
+template<typename InputIterator>
+static void pushRequestOption
+(BDE& dict, InputIterator optionFirst, InputIterator optionLast)
+{
+  const std::set<std::string>& requestOptions = listRequestOptions();
+  for(; optionFirst != optionLast; ++optionFirst) {
+    if(requestOptions.count((*optionFirst).first)) {
+      dict[(*optionFirst).first] = (*optionFirst).second;
+    }
+  }
+}
+
+BDE GetOptionXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+  if(params.empty() || !params[0].isString()) {
+    throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
+  }  
+  int32_t gid = util::parseInt(params[0].s());
+
+  SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
+  if(group.isNull()) {
+    throw DL_ABORT_EX
+      (StringFormat("Cannot get option for GID#%d", gid).str());
+  }
+  BDE result = BDE::dict();
+  SharedHandle<Option> option = group->getOption();
+  pushRequestOption(result, option->begin(), option->end());
+  return result;
+}
+
+BDE GetGlobalOptionXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  BDE result = BDE::dict();
+  for(std::map<std::string, std::string>::const_iterator i = e->option->begin();
+      i != e->option->end(); ++i) {
+    SharedHandle<OptionHandler> h = _optionParser->findByName((*i).first);
+    if(!h.isNull() && !h->isHidden()) {
+      result[(*i).first] = (*i).second;
+    }
+  }
+  return result;
+}
+
+BDE ChangePositionXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+
+  if(params.size() != 3 ||
+     !params[0].isString() || !params[1].isInteger() || !params[2].isString()) {
+    throw DL_ABORT_EX("Illegal argument.");
+  }
+  int32_t gid = util::parseInt(params[0].s());
+  int pos = params[1].i();
+  const std::string& howStr = params[2].s();
+  RequestGroupMan::HOW how;
+  if(howStr == "POS_SET") {
+    how = RequestGroupMan::POS_SET;
+  } else if(howStr == "POS_CUR") {
+    how = RequestGroupMan::POS_CUR;
+  } else if(howStr == "POS_END") {
+    how = RequestGroupMan::POS_END;
+  } else {
+    throw DL_ABORT_EX("Illegal argument.");
+  }
+  size_t destPos =
+    e->_requestGroupMan->changeReservedGroupPosition(gid, pos, how);
+  BDE result(destPos);
+  return result;
+}
+
+BDE SystemMulticallXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+  
+  if(params.size() != 1) {
+    throw DL_ABORT_EX("Illegal argument. One item list is expected.");
+  }
+  const BDE& methodSpecs = params[0];
+  BDE list = BDE::list();
+  for(BDE::List::const_iterator i = methodSpecs.listBegin();
+      i != methodSpecs.listEnd(); ++i) {
+    if(!(*i).isDict()) {
+      list << createErrorResponse
+	(DL_ABORT_EX("system.multicall expected struct."));
+      continue;
+    }
+    if(!(*i).containsKey(KEY_METHOD_NAME) ||
+       !(*i).containsKey(KEY_PARAMS)) {
+      list << createErrorResponse
+	(DL_ABORT_EX("Missing methodName or params."));
+      continue;
+    }
+    const std::string& methodName = (*i)[KEY_METHOD_NAME].s();
+    if(methodName == getMethodName()) {
+      list << createErrorResponse
+	(DL_ABORT_EX("Recursive system.multicall forbidden."));
+      continue;
+    }
+    SharedHandle<XmlRpcMethod> method = XmlRpcMethodFactory::create(methodName);
+    XmlRpcRequest innerReq(methodName, (*i)[KEY_PARAMS]);
+    XmlRpcResponse res = method->execute(innerReq, e);
+    if(res._code == 0) {
+      BDE l = BDE::list();
+      l << res._param;
+      list << l;
+    } else {
+      list << res._param;
+    }
+  }
+  return list;
+}
+
 BDE NoSuchMethodXmlRpcMethod::process
 (const XmlRpcRequest& req, DownloadEngine* e)
 {
