@@ -45,7 +45,7 @@
 #include "OptionParser.h"
 #include "OptionHandlerFactory.h"
 #include "OptionHandler.h"
-#include "Util.h"
+#include "util.h"
 #include "message.h"
 #include "Exception.h"
 #include "a2io.h"
@@ -53,7 +53,12 @@
 #include "File.h"
 #include "StringFormat.h"
 #include "OptionHandlerException.h"
-#include "DownloadResult.h"
+#include "DownloadResultCode.h"
+#include "SimpleRandomizer.h"
+#include "bittorrent_helper.h"
+#ifndef HAVE_DAEMON
+#include "daemon.h"
+#endif // !HAVE_DAEMON
 
 namespace aria2 {
 
@@ -96,7 +101,7 @@ void option_processing(Option& op, std::deque<std::string>& uris,
 
       if(op.defined("version")) {
 	showVersion();
-	exit(DownloadResult::FINISHED);
+	exit(downloadresultcode::FINISHED);
       }
       if(op.defined("help")) {
 	std::string keyword;
@@ -104,7 +109,7 @@ void option_processing(Option& op, std::deque<std::string>& uris,
 	  keyword = TAG_BASIC;
 	} else {
 	  keyword = op.get("help");
-	  if(Util::startsWith(keyword, "--")) {
+	  if(util::startsWith(keyword, "--")) {
 	    keyword = keyword.substr(2);
 	  }
 	  std::string::size_type eqpos = keyword.find("=");
@@ -113,7 +118,7 @@ void option_processing(Option& op, std::deque<std::string>& uris,
 	  }
 	}
 	showUsage(keyword, oparser);
-	exit(DownloadResult::FINISHED);
+	exit(downloadresultcode::FINISHED);
       }
     }
 
@@ -135,18 +140,18 @@ void option_processing(Option& op, std::deque<std::string>& uris,
 		    << "Usage:" << "\n"
 		    << oparser.findByName(e.getOptionName())->getDescription()
 		    << std::endl;
-	  exit(DownloadResult::UNKNOWN_ERROR);
+	  exit(downloadresultcode::UNKNOWN_ERROR);
 	} catch(Exception& e) {
 	  std::cerr << "Parse error in " << cfname << "\n"
 		    << e.stackTrace() << std::endl;
-	  exit(DownloadResult::UNKNOWN_ERROR);
+	  exit(downloadresultcode::UNKNOWN_ERROR);
 	}
       } else if(!ucfname.empty()) {
 	std::cerr << StringFormat("Configuration file %s is not found.",
 				  cfname.c_str())
 		  << "\n";
 	showUsage(TAG_HELP, oparser);
-	exit(DownloadResult::UNKNOWN_ERROR);
+	exit(downloadresultcode::UNKNOWN_ERROR);
       }
     }
     // Override configuration with environment variables.
@@ -166,11 +171,11 @@ void option_processing(Option& op, std::deque<std::string>& uris,
 	      << "Usage:" << "\n"
 	      << oparser.findByName(e.getOptionName())
 	      << std::endl;
-    exit(DownloadResult::UNKNOWN_ERROR);
+    exit(downloadresultcode::UNKNOWN_ERROR);
   } catch(Exception& e) {
     std::cerr << e.stackTrace() << std::endl;
     showUsage(TAG_HELP, oparser);
-    exit(DownloadResult::UNKNOWN_ERROR);
+    exit(downloadresultcode::UNKNOWN_ERROR);
   }
   if(
 #ifdef ENABLE_XML_RPC
@@ -186,17 +191,15 @@ void option_processing(Option& op, std::deque<std::string>& uris,
     if(uris.empty()) {
       std::cerr << MSG_URI_REQUIRED << std::endl;
       showUsage(TAG_HELP, oparser);
-      exit(DownloadResult::UNKNOWN_ERROR);
+      exit(downloadresultcode::UNKNOWN_ERROR);
     }
   }
-#ifdef HAVE_DAEMON
   if(op.getAsBool(PREF_DAEMON)) {
-    if(daemon(1, 1) < 0) {
+    if(daemon(0, 0) < 0) {
       perror(MSG_DAEMON_FAILED);
-      exit(DownloadResult::UNKNOWN_ERROR);
+      exit(downloadresultcode::UNKNOWN_ERROR);
     }
   }
-#endif // HAVE_DAEMON
 }
 
 } // namespace aria2
