@@ -52,8 +52,6 @@ extern const std::string V_ALL;
  * General preferences
  */
 // values: 1*digit
-extern const std::string PREF_RETRY_WAIT;
-// values: 1*digit
 extern const std::string PREF_TIMEOUT;
 // values: 1*digit
 extern const std::string PREF_DNS_TIMEOUT;
@@ -83,9 +81,10 @@ extern const std::string PREF_SEGMENT_SIZE;
 extern const std::string PREF_MAX_DOWNLOAD_LIMIT;
 // value: 1*digit
 extern const std::string PREF_STARTUP_IDLE_TIME;
-// value: prealloc | none
+// value: prealloc | falloc | none
 extern const std::string PREF_FILE_ALLOCATION;
 extern const std::string V_PREALLOC;
+extern const std::string V_FALLOC;
 // value: 1*digit
 extern const std::string PREF_NO_FILE_ALLOCATION_LIMIT;
 // value: true | false
@@ -101,6 +100,8 @@ extern const std::string PREF_CONTINUE;
 // value:
 extern const std::string PREF_NO_NETRC;
 // value: 1*digit
+extern const std::string PREF_MAX_OVERALL_DOWNLOAD_LIMIT;
+// value: 1*digit
 extern const std::string PREF_MAX_DOWNLOADS;
 // value: string that your file system recognizes as a file name.
 extern const std::string PREF_INPUT_FILE;
@@ -135,10 +136,11 @@ extern const std::string V_INFO;
 extern const std::string V_NOTICE;
 extern const std::string V_WARN;
 extern const std::string V_ERROR;
-// value: inorder | feedback
+// value: inorder | feedback | adaptive
 extern const std::string PREF_URI_SELECTOR;
 extern const std::string V_INORDER;
 extern const std::string V_FEEDBACK;
+extern const std::string V_ADAPTIVE;
 // value: 1*digit
 extern const std::string PREF_SERVER_STAT_TIMEOUT;
 // value: string that your file system recognizes as a file name.
@@ -149,6 +151,35 @@ extern const std::string PREF_SERVER_STAT_OF;
 extern const std::string PREF_REMOTE_TIME;
 // value: 1*digit
 extern const std::string PREF_MAX_FILE_NOT_FOUND;
+// value: epoll | select
+extern const std::string PREF_EVENT_POLL;
+extern const std::string V_EPOLL;
+extern const std::string V_SELECT;
+// value: 1*digit
+extern const std::string PREF_XML_RPC_LISTEN_PORT;
+// value: true | false
+extern const std::string PREF_ENABLE_XML_RPC;
+// value: true | false
+extern const std::string PREF_DRY_RUN;
+// value: true | false
+extern const std::string PREF_REUSE_URI;
+// value: string
+extern const std::string PREF_XML_RPC_USER;
+// value: string
+extern const std::string PREF_XML_RPC_PASSWD;
+// value: 1*digit
+extern const std::string PREF_XML_RPC_MAX_REQUEST_SIZE;
+// value: string
+extern const std::string PREF_ON_DOWNLOAD_START;
+extern const std::string PREF_ON_DOWNLOAD_STOP;
+extern const std::string PREF_ON_DOWNLOAD_COMPLETE;
+extern const std::string PREF_ON_DOWNLOAD_ERROR;
+// value: true | false
+extern const std::string PREF_XML_RPC_LISTEN_ALL;
+// value: string
+extern const std::string PREF_INTERFACE;
+// value: true | false
+extern const std::string PREF_DISABLE_IPV6;
 
 /**
  * FTP related preferences
@@ -169,13 +200,12 @@ extern const std::string PREF_FTP_REUSE_CONNECTION;
  */
 extern const std::string PREF_HTTP_USER;
 extern const std::string PREF_HTTP_PASSWD;
-// values: basic
-extern const std::string PREF_HTTP_AUTH_SCHEME;
-extern const std::string V_BASIC;
 // values: string
 extern const std::string PREF_USER_AGENT;
 // value: string that your file system recognizes as a file name.
 extern const std::string PREF_LOAD_COOKIES;
+// value: string that your file system recognizes as a file name.
+extern const std::string PREF_SAVE_COOKIES;
 // values: true | false
 extern const std::string PREF_ENABLE_HTTP_KEEP_ALIVE;
 // values: true | false
@@ -194,6 +224,8 @@ extern const std::string PREF_CA_CERTIFICATE;
 extern const std::string PREF_CHECK_CERTIFICATE;
 // value: true | false
 extern const std::string PREF_USE_HEAD;
+// value: true | false
+extern const std::string PREF_HTTP_AUTH_CHALLENGE;
 
 /**;
  * Proxy related preferences
@@ -220,8 +252,6 @@ extern const std::string PREF_BT_TIMEOUT;
 extern const std::string PREF_BT_REQUEST_TIMEOUT;
 // values: true | false
 extern const std::string PREF_SHOW_FILES;
-// values: true | false
-extern const std::string PREF_DIRECT_FILE_MAPPING;
 // values: 1*digit
 extern const std::string PREF_MAX_OVERALL_UPLOAD_LIMIT;
 // values: 1*digit
@@ -268,6 +298,22 @@ extern const std::string PREF_BT_REQUEST_PEER_SPEED_LIMIT;
 extern const std::string PREF_BT_MAX_OPEN_FILES;
 // values: true | false
 extern const std::string PREF_BT_SEED_UNVERIFIED;
+// values: true | false
+extern const std::string PREF_BT_HASH_CHECK_SEED;
+// values: 1*digit
+extern const std::string PREF_BT_MAX_PEERS;
+// values: a string (IP address)
+extern const std::string PREF_BT_EXTERNAL_IP;
+// values: 1*digit '=' a string that your file system recognizes as a file name.
+extern const std::string PREF_INDEX_OUT;
+// values: 1*digit
+extern const std::string PREF_BT_TRACKER_INTERVAL;
+// values: 1*digit
+extern const std::string PREF_BT_STOP_TIMEOUT;
+// values: head[=SIZE]|tail[=SIZE], ...
+extern const std::string PREF_BT_PRIORITIZE_PIECE;
+// values: true | false
+extern const std::string PREF_BT_SAVE_METADATA;
 
 /**
  * Metalink related preferences
