@@ -4,9 +4,8 @@
 
 #include <cppunit/extensions/HelperMacros.h>
 
-#include "PeerMessageUtil.h"
+#include "bittorrent_helper.h"
 #include "MockBtMessageDispatcher.h"
-#include "MockBtContext.h"
 #include "Peer.h"
 #include "FileEntry.h"
 #include "Piece.h"
@@ -22,12 +21,9 @@ class BtCancelMessageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST_SUITE_END();
 private:
   SharedHandle<Peer> peer;
-  SharedHandle<MockBtContext> btContext;
 public:
   void setUp() {
     peer.reset(new Peer("host", 6969));
-    btContext.reset(new MockBtContext());
-    btContext->setInfoHash((const unsigned char*)"12345678901234567890");
   }
 
   void testCreate();
@@ -57,10 +53,10 @@ CPPUNIT_TEST_SUITE_REGISTRATION(BtCancelMessageTest);
 
 void BtCancelMessageTest::testCreate() {
   unsigned char msg[17];
-  PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 13, 8);
-  PeerMessageUtil::setIntParam(&msg[5], 12345);
-  PeerMessageUtil::setIntParam(&msg[9], 256);
-  PeerMessageUtil::setIntParam(&msg[13], 1024);
+  bittorrent::createPeerMessageString(msg, sizeof(msg), 13, 8);
+  bittorrent::setIntParam(&msg[5], 12345);
+  bittorrent::setIntParam(&msg[9], 256);
+  bittorrent::setIntParam(&msg[13], 1024);
   SharedHandle<BtCancelMessage> pm = BtCancelMessage::create(&msg[4], 13);
   CPPUNIT_ASSERT_EQUAL((uint8_t)8, pm->getId());
   CPPUNIT_ASSERT_EQUAL((size_t)12345, pm->getIndex());
@@ -70,7 +66,7 @@ void BtCancelMessageTest::testCreate() {
   // case: payload size is wrong
   try {
     unsigned char msg[18];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 14, 8);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 14, 8);
     BtCancelMessage::create(&msg[4], 14);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -78,7 +74,7 @@ void BtCancelMessageTest::testCreate() {
   // case: id is wrong
   try {
     unsigned char msg[17];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 13, 9);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 13, 9);
     BtCancelMessage::create(&msg[4], 13);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -91,10 +87,10 @@ void BtCancelMessageTest::testGetMessage() {
   msg.setBegin(256);
   msg.setLength(1024);
   unsigned char data[17];
-  PeerMessageUtil::createPeerMessageString(data, sizeof(data), 13, 8);
-  PeerMessageUtil::setIntParam(&data[5], 12345);
-  PeerMessageUtil::setIntParam(&data[9], 256);
-  PeerMessageUtil::setIntParam(&data[13], 1024);
+  bittorrent::createPeerMessageString(data, sizeof(data), 13, 8);
+  bittorrent::setIntParam(&data[5], 12345);
+  bittorrent::setIntParam(&data[9], 256);
+  bittorrent::setIntParam(&data[13], 1024);
   CPPUNIT_ASSERT(memcmp(msg.getMessage(), data, 17) == 0);
 }
 
@@ -103,7 +99,6 @@ void BtCancelMessageTest::testDoReceivedAction() {
   msg.setIndex(1);
   msg.setBegin(2*16*1024);
   msg.setLength(16*1024);
-  msg.setBtContext(btContext);
   msg.setPeer(peer);
   SharedHandle<MockBtMessageDispatcher2> dispatcher
     (new MockBtMessageDispatcher2());
