@@ -1,8 +1,14 @@
 #include "BtInterestedMessage.h"
-#include "PeerMessageUtil.h"
+
+#include <cstring>
+
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+#include "bittorrent_helper.h"
+#include "Peer.h"
+#include "MockPeerStorage.h"
+
+namespace aria2 {
 
 class BtInterestedMessageTest:public CppUnit::TestFixture {
 
@@ -13,12 +19,7 @@ class BtInterestedMessageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testOnSendComplete);
   CPPUNIT_TEST(testToString);
   CPPUNIT_TEST_SUITE_END();
-private:
-
 public:
-  void setUp() {
-  }
-
   void testCreate();
   void testGetMessage();
   void testDoReceivedAction();
@@ -31,14 +32,14 @@ CPPUNIT_TEST_SUITE_REGISTRATION(BtInterestedMessageTest);
 
 void BtInterestedMessageTest::testCreate() {
   unsigned char msg[5];
-  PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 1, 2);
+  bittorrent::createPeerMessageString(msg, sizeof(msg), 1, 2);
   BtInterestedMessageHandle pm = BtInterestedMessage::create(&msg[4], 1);
-  CPPUNIT_ASSERT_EQUAL((int8_t)2, pm->getId());
+  CPPUNIT_ASSERT_EQUAL((uint8_t)2, pm->getId());
 
   // case: payload size is wrong
   try {
     unsigned char msg[6];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 2, 2);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 2, 2);
     BtInterestedMessage::create(&msg[4], 2);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -46,7 +47,7 @@ void BtInterestedMessageTest::testCreate() {
   // case: id is wrong
   try {
     unsigned char msg[5];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 1, 3);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 1, 3);
     BtInterestedMessage::create(&msg[4], 1);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -56,29 +57,43 @@ void BtInterestedMessageTest::testCreate() {
 void BtInterestedMessageTest::testGetMessage() {
   BtInterestedMessage msg;
   unsigned char data[5];
-  PeerMessageUtil::createPeerMessageString(data, sizeof(data), 1, 2);
+  bittorrent::createPeerMessageString(data, sizeof(data), 1, 2);
   CPPUNIT_ASSERT(memcmp(msg.getMessage(), data, 5) == 0);
 }
 
 void BtInterestedMessageTest::testDoReceivedAction() {
   BtInterestedMessage msg;
-  PeerHandle peer = new Peer("host", 6969, 16*1024, 256*1024);
+  SharedHandle<Peer> peer(new Peer("host", 6969));
+  peer->allocateSessionResource(1024, 1024*1024);
   msg.setPeer(peer);
-  CPPUNIT_ASSERT(!peer->peerInterested);
+
+  SharedHandle<MockPeerStorage> peerStorage(new MockPeerStorage());
+
+  msg.setPeerStorage(peerStorage);
+
+  CPPUNIT_ASSERT(!peer->peerInterested());
   msg.doReceivedAction();
-  CPPUNIT_ASSERT(peer->peerInterested);
+  CPPUNIT_ASSERT(peer->peerInterested());
+  CPPUNIT_ASSERT_EQUAL(0, peerStorage->getNumChokeExecuted());
+
+  peer->amChoking(false);
+  msg.doReceivedAction();
+  CPPUNIT_ASSERT_EQUAL(1, peerStorage->getNumChokeExecuted());
 }
 
 void BtInterestedMessageTest::testOnSendComplete() {
   BtInterestedMessage msg;
-  PeerHandle peer = new Peer("host", 6969, 16*1024, 256*1024);
+  SharedHandle<Peer> peer(new Peer("host", 6969));
+  peer->allocateSessionResource(1024, 1024*1024);
   msg.setPeer(peer);
-  CPPUNIT_ASSERT(!peer->amInterested);
+  CPPUNIT_ASSERT(!peer->amInterested());
   msg.onSendComplete();
-  CPPUNIT_ASSERT(peer->amInterested);
+  CPPUNIT_ASSERT(peer->amInterested());
 }
 
 void BtInterestedMessageTest::testToString() {
   BtInterestedMessage msg;
-  CPPUNIT_ASSERT_EQUAL(string("interested"), msg.toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("interested"), msg.toString());
 }
+
+} // namespace aria2
