@@ -13,6 +13,7 @@
 #include "DiskAdaptor.h"
 #include "util.h"
 #include "MessageDigestHelper.h"
+#include "prefs.h"
 
 namespace aria2 {
 
@@ -31,6 +32,7 @@ public:
   {
     _option.reset(new Option());
     _option->put("HELLO", "WORLD");
+    _option->put(PREF_DIR, ".");
     _dctx.reset(new DownloadContext(0, 0, "something"));
     _requestGroup.reset(new RequestGroup(_option));
     _requestGroup->setDownloadContext(_dctx);
@@ -62,6 +64,10 @@ void UTMetadataPostDownloadHandlerTest::testCanHandle()
 
 void UTMetadataPostDownloadHandlerTest::testGetNextRequestGroups()
 {
+  File trfile("cd41c7fdddfd034a15a04d7ff881216e01c4ceaf.torrent");
+  if(trfile.exists()) {
+    trfile.remove();
+  }
   std::string metadata =
     "d6:lengthi384e4:name19:aria2-0.8.2.tar.bz212:piece lengthi128e"
     "6:pieces60:AAAAAAAAAAAAAAAAAAAABBBBBBBBBBBBBBBBBBBBCCCCCCCCCCCCCCCCCCCCe";
@@ -100,6 +106,17 @@ void UTMetadataPostDownloadHandlerTest::testGetNextRequestGroups()
 		       newAttrs[bittorrent::ANNOUNCE_LIST][0][0].s());
   CPPUNIT_ASSERT_EQUAL(_option->get("Hello"),
 		       newRg->getOption()->get("Hello"));
+  CPPUNIT_ASSERT
+    (std::find(_requestGroup->followedBy().begin(),
+	       _requestGroup->followedBy().end(),
+	       newRg->getGID()) != _requestGroup->followedBy().end());
+  CPPUNIT_ASSERT(!trfile.exists());
+
+  results.clear();
+
+  _requestGroup->getOption()->put(PREF_BT_SAVE_METADATA, V_TRUE);
+  handler.getNextRequestGroups(results, _requestGroup.get());
+  CPPUNIT_ASSERT(trfile.exists());
 
   results.clear();
 
@@ -109,8 +126,12 @@ void UTMetadataPostDownloadHandlerTest::testGetNextRequestGroups()
   _requestGroup->getPieceStorage()->getDiskAdaptor()->writeData
     (reinterpret_cast<const unsigned char*>(metadata.data()), metadata.size(),
      0);
-  handler.getNextRequestGroups(results, _requestGroup.get());
-  CPPUNIT_ASSERT(results.empty());  
+  try {
+    handler.getNextRequestGroups(results, _requestGroup.get());
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(RecoverableException& e) {
+    // success
+  }
 }
 
 } // namespace aria2
